/**
 * ZnetDK, Starter Web Application for rapid & easy development
 * See official website http://www.znetdk.fr
 * Copyright (C) 2019 Pascal MARTINEZ (contact@znetdk.fr)
 * License GNU GPL http://www.gnu.org/licenses/gpl-3.0.html GNU GPL
 * --------------------------------------------------------------------
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * --------------------------------------------------------------------
 * ZnetDK Javascript library for mobile page layout
 *
 * File version: 1.5
 * Last update: 05/27/2022
 */

/* global FormData */


var znetdkMobile = {
    ajax: {
        requestContext: [],
        loaderElementId: "zdk-ajax-loading",
        requestInProgress: 0
    },
    authentication: {
        loginFormId: '#zdk-login-modal',
        changePasswordFormId: '#zdk-changepwd-modal',
        connectedUserPanelId: '#zdk-userpanel-modal',
        rememberMeLocalStorageKey: 'znetdkmobile_authentication_remember_me'
    },
    browser: {
        confirmationOnApplicationClose: true,
        events: {}
    },
    header: {
        headerId: '#zdk-header',
        menuButtonId: '#zdk-side-nav-button',
        connectionAreaId: '#zdk-connection-area',
        autoHideOnScrollEnabled: false,
        events: {}
    },
    navigation: {
        companyLogoId: '#zdk-company-logo',
        menuDefinitionId: '#zdk-custom-menu',
        verticalMenuId: '#zdk-side-nav-menu',
        horizontalMenuId: '#zdk-tab-menu',
        verticalMenuActiveItemClass: 'w3-theme-l2',
        horizontalMenuActiveItemClass: 'w3-theme-l2',
        horizontalMenuItemBorderClass: 'w3-border-theme',
        horizontalMenuActiveItemBorderClass: 'w3-border-red',
        autoReloadViewClass: 'zdk-viewreload',
        events: {
            beforeViewDisplayName: 'beforeviewdisplay',
            afterViewDisplayName: 'afterviewdisplay'
        }
    },
    content: {
        containerId: '#zdk-content',
        defaultContainerId: '#default_content'
    },
    footer: {
        footerId: '#zdk-footer'
    },
    messages: {
        element: null,
        containerId: '#zdk-messages',
        colors: {
            info: 'w3-blue',
            warn: 'w3-yellow',
            error: 'w3-red',
            critical: 'w3-blue-grey'
        },
        icons: {
            info: 'fa-info-circle',
            warn: 'fa-warning',
            error: 'fa-times-circle',
            critical: 'fa-ban'
        },
        autoCloseDuration: 6000,
        events: {}
    },
    log: {},
    serviceWorker: {},
    modal: {
        element: null,
        cssClass: 'w3-modal',
        closeOnSubmitSuccess: true,
        events: {
            beforeCloseNane: 'beforemodalclose'
        }
    },
    form: {
        element: null,
        inputData: null,
        remoteActions: {
            submit: {
                controller: null,
                action: null
            },
            load: {
                controller: null,
                action: null
            }
        },
        events: {
            afterSubmitSuccessName: 'aftersubmitsuccess'
        }
    },
    action: {
        buttonGap: 60,
        buttons: {
            add: {id: '#zdk-mobile-action-add'},
            refresh: {id: '#zdk-mobile-action-refresh'},
            search: {id: '#zdk-mobile-action-search'},
            scrollup: {id: '#zdk-mobile-action-scrollup'}
        },
        views: {},
        events: {}
    },
    list: {
        element: null,
        rowTemplate: null,
        noRowMessage: '<li><h3>No row found!</h3></li>',
        customSortCriteria: null,
        defaultCustomSortCriterium: null,
        rowsPerPage: 20,
        heightDiffForNewPage: 200,
        isAutocompleteOnSearchEnabled: false,
        uniqueSearchedKeyword: false,
        searchedKeywordAsJson: false,
        beforeSearchRequestCallback: null,
        searchModalId: '#zdk-searchinlist-modal',
        sortFieldStorageKey: 'znetdkmobile_list_sort_field',
        sortOrderStorageKey: 'znetdkmobile_list_sort_order',
        sortLabelStorageKey: 'znetdkmobile_list_sort_label',
        keywordsStorageKey: 'znetdkmobile_list_filter_keywords',
        remoteActions: {
            load: {
                controller: null,
                action: null
            },
            autocomplete: {
                controller: null,
                action: null
            }
        },
        events: {
            afterClickEditButtonName: 'afterClickEdit',
            onContentScrollName: 'onContentScroll'
        }
    },
    autocomplete: {
        element: null,
        minStringLength: 1,
        delay: 300,
        listTemplate: '<ul class="autocomplete w3-ul w3-card w3-hoverable w3-hide"/>',
        itemTemplate: '<li class="w3-hover-theme"/>',
        selectedItemCssClass: 'w3-theme-dark',
        remoteActions: {
            autocomplete: {
                controller: null,
                action: null
            }
        }
    },
    file: {}

};

//******************************* INITIALIZATION *******************************

/**
 * The page is fully loaded
 */
$(document).ready(function () {
    znetdkMobile.init();
});

/**
 * Initialize the application once the main HTML page is loaded
 * - The login form is displayed if authentication is required
 * - The vertical and horizontal menus are built from the menu definition found
 * into the HTML page
 * - The embedded view or the view matching the first menu item is displayed
 * - The events handled by the application are attached to the built-in event
 *  handlers
 */
znetdkMobile.init = function () {
    if (znetdkMobile.authentication.isRequired()) { // Authentication is required
        // No navigation available
        znetdkMobile.navigation.setNoNavigation();
        // The footer is displayed at the bottom of the viewport
        znetdkMobile.footer.adjustPosition();
        // Show login form
        znetdkMobile.authentication.showLoginForm();
    } else { // No Authentication required (user authenticated or authentication disabled
        // The footer is temporarily hidden
        znetdkMobile.footer.hide();
        // Building of the navigation menus
        znetdkMobile.navigation.build();
        // Is authentication enabled ?
        if (znetdkMobile.authentication.isEnabled()) {
            // The header connected user buttons are displayed
            znetdkMobile.header.showConnectionArea();
        } else {
            znetdkMobile.header.hideConnectionArea();
        }
    }
    // Handle all events
    this.browser.events.handleViewportResize(); // Viewport resize
    this.browser.events.handleBeforeUnload();// Confirmation on application close
    this.modal.events.handleAllClose(); // Close modal dialog
    this.form.events.handleAllInput(); // Form input data modified by user
    this.form.events.handleAllSubmit(); // Submit a data form
    this.form.events.handleTogglePassword(); // Show/hide password
    this.messages.events.handleAllClose(); // Close a message panel
    this.action.events.handleClick(); // Click on action buttons
    this.list.events.handleEdit(); // Click on list edit buttons
    this.list.events.handleScroll(); // Scroll of the list items
    if (znetdkMobile.authentication.isEnabled()) {
        this.header.events.handleProfileButtonClick(); // Heading Profile button
        this.header.events.handleLogoutButtonClick(); // Logout button
    }
    // Service Worker registration
    this.serviceWorker.register();
};

//***************************** AJAX PUBLIC METHODS ****************************

/**
 * Returns the Ajax URL and its parameter as an Object
 * @returns {Object} The AJAX URL as properties:
 * url: the URL without parameters
 * paramName: the parameter name
 * paramValue: the parameter value
 */
znetdkMobile.ajax.getParamsFromAjaxURL = function() {
    var requestUrl = $('body').data('ajaxurl'),
            URLArray = requestUrl.split("?"),
            paramArray = [],
            result = {};
    result.url = URLArray[0];
    if (URLArray.length === 2) {
        paramArray = URLArray[1].split("=");
        result.paramName = paramArray[0];
        result.paramValue = paramArray[1];
    }
    return result;
};

/**
 * Send an AJAX request to a ZnetDK PHP application controller
 * @param {Object} options The request parameters:
 * controller: the application controller name (mandatory)
 * action: the application controller action name (mandatory)
 * data: the data as JS Object to send to the controller action (optional)
 * callback: the function to callback when request succeeded
 * htmlTarget: the HTML element as jQuery object in which the request response
 * is to append to.
 * errorCallback: the function to call back when request failed
 * @returns {jqXHR|Boolean} The jQuery object returned by the jQuery $.ajax
 * method or false if the control and action properties are not set properly.
 */
znetdkMobile.ajax.request = function (options) {
    if (options.hasOwnProperty('controller') && !options.hasOwnProperty('control')) {
        options.control = options.controller; // 'controller' property also supported
    }
    if (options.control && options.action) {
        var ajaxURL = this.getParamsFromAjaxURL();
        var ajaxOptions = {
            type: "POST",
            url: ajaxURL.url,
            //******************* AJAX EVENT HANDLERS **************************
            /**
             * Event Handler of an AJAX request when sent.
             * The Ajax Loader image and overlay are added to the body
             */
            beforeSend: function () {
                if (!znetdkMobile.ajax.requestInProgress) {
                    var loaderDefinition = $('body').data('ajaxloader'),
                            loaderElement = $(loaderDefinition);
                    loaderElement.attr('id', znetdkMobile.ajax.loaderElementId);
                    $('body').append(loaderElement);
                    $('body').append('<div id="zdkmobile-ajax-loading-overlay"/>');
                }
                znetdkMobile.ajax.requestInProgress += 1;
            },
            /**
             * Event Handler of an AJAX request when succeeded.
             * If the AJAX response is a view, it is added to the end of the target HTML
             * element.
             * If the AJAX response is an object, it is passed to the callback function.
             * The callback function when specified as parameter of the AJAX request is
             * called.
             * An error message is displayed if the response is invalid.
             * @param {String|Object} response The response of the remote controller action
             */
            success: function (response) {
                try {
                    if (options.htmlTarget) {
                        options.htmlTarget.prepend(response);
                        if (typeof options.callback === "function") {
                            options.callback(response);
                        }
                    } else if (typeof response === "object") {
                        if (typeof options.callback === "function") {
                            options.callback(response);
                        } else if (response.hasOwnProperty('msg')
                                && (response.hasOwnProperty('success')
                                        || response.hasOwnProperty('warning'))) {
                            var levelMsg = response.success === false ? 'error' :
                                    response.warning === true ? 'warn' : 'info',
                                    summary = response.hasOwnProperty('summary') ? response.summary : 'Message';
                            if (levelMsg !== 'error' && (!response.hasOwnProperty('summary')
                                    || response.summary === null || response.summary === '')) {
                                // Message displayed as Snackbar if the summary is missing and
                                // the message level is not 'error'
                                znetdkMobile.messages.showSnackbar(response.msg, levelMsg === 'warn');
                            } else {
                                znetdkMobile.messages.add(levelMsg, summary, response.msg, false);
                            }
                        }
                    } else {
                        znetdkMobile.messages.add('error', 'Message',
                                'Invalid Ajax response sent by the web server!', false);
                    }
                } catch (error) {
                    hideLoaderImage();
                    znetdkMobile.navigation.closeVerticalMenu();
                    znetdkMobile.messages.add('critical', 'OOPS!',
                            'A Javascript error occured while processing the AJAX request response! See your browser console for details.');
                    znetdkMobile.log.error("Javascript error dectected while inserting the view '"
                            + options.control + "' in the DOM. Message: " + error);
                }
            },
            /**
             * Event Handler of an AJAX request when failed.
             * The error message is displayed.
             * When the HTTP error code is 401, the AJAX request is queued and the login
             * form is displayed.
             * If the option.errorCallback is set, the function is called back when the
             * the HTTP error code is different from 401.
             * @param {Object} response The error informations
             */
            error: function (response) {
                var errorMsg, errorSummary, appVersion, reloadSummary, reloadMsg,
                        errorLevel = response.status === 401 ? 'warn' : 'critical';
                if (/application\/json/.test(response.getResponseHeader('Content-Type'))) {
                    try {
                        var errorObject = JSON.parse(response.responseText);
                        errorMsg = errorObject.msg;
                        errorSummary = errorObject.summary;
                        appVersion = errorObject.appver;
                        reloadSummary = errorObject.reload_summary;
                        reloadMsg = errorObject.reload_msg;
                    } catch (e) {
                    }
                }
                if (response.status === 0) {
                    var msgArray = $('body').data('networkerrormsg').split("|");
                    errorSummary = msgArray[0];
                    errorMsg = msgArray[1];
                    errorLevel = 'error';
                } else if (typeof errorMsg === 'undefined') {
                    errorMsg = "The JSON response returned by the controller='" + options.control + "' and the action='" + options.action +
                            "' can't be parsed! HTTP status: " + response.status + ' ' + response.statusText;
                    errorSummary = 'Error parsing server response';
                }
                if (typeof options.errorCallback === "function"
                        && options.errorCallback(response) === false) {
                    return;
                }
                if (response.status === 401) {
                    // Show session timed out message as snack bar
                    znetdkMobile.messages.showSnackbar(errorMsg, true);
                    if (typeof appVersion === 'number' && typeof reloadSummary === 'string'
                            && typeof reloadMsg === 'string'
                            && parseInt($('body').data('appver')) < appVersion) {
                        // New app's version, reloading is required
                        znetdkMobile.messages.notify(reloadSummary, reloadMsg, 'Ok', function(){
                            // No longer confirmation message on application reload or close
                            znetdkMobile.browser.events.detachBeforeUnload();
                             // The page is reloaded
                            location.reload();
                        });
                    } else {
                        // Current request is queued for execution after authentication
                        znetdkMobile.ajax.requestContext.push(options);
                        if (znetdkMobile.ajax.requestContext.length === 1) {
                            // Login dialog to renew user credentials
                            znetdkMobile.authentication.showLoginForm(true);
                        }
                    }
                } else {
                    znetdkMobile.messages.add(errorLevel, errorSummary, errorMsg, false);
                }
            },
            /**
             * Event Handler of an AJAX request when terminated.
             * The AJAX Loader image is hidden
             */
            complete: function () {
                hideLoaderImage();
            }
        };
        if (options.data && options.data instanceof FormData) {
            var requestData = options.data;
            requestData.append("control", options.control);
            requestData.append("action", options.action);
            if (ajaxURL.hasOwnProperty('paramName')) {
                requestData.append(ajaxURL.paramName, ajaxURL.paramValue);
            }
            ajaxOptions.data = requestData;
            ajaxOptions.processData = false;
            ajaxOptions.contentType = false;
        } else if (options.data && typeof options.data === "object") {
            var requestData;
            if (Array.isArray(options.data)) {
                requestData = 'control=' + options.control + '&action=' + options.action + '&' + $.param(options.data);
            } else { // Is Object...
                var property;
                requestData = 'control=' + options.control + '&action=' + options.action;
                for (property in options.data) {
                    var value = options.data[property],
                            encodedValue = typeof value === 'string' ? encodeURIComponent(value) : value;
                    if (typeof value !== 'undefined' && value !== null) {
                        requestData += '&' + property + '=' + encodedValue;
                    }
                }
            }
            if (ajaxURL.hasOwnProperty('paramName')) {
                requestData += '&' + ajaxURL.paramName + '=' + ajaxURL.paramValue;
            }
            ajaxOptions.data = requestData;
        } else {
            ajaxOptions.data = {control: options.control, action: options.action};
            if (ajaxURL.hasOwnProperty('paramName')) {
                ajaxOptions.data[ajaxURL.paramName] = ajaxURL.paramValue;
            }
        }
        return $.ajax(ajaxOptions);
    }
    znetdkMobile.log.error("Call to the request method failed, properties 'control' or 'action' are missing: ", options);
    return false;
    /**
     * The Ajax Loader image and overlay are removed from the body
     */
    function hideLoaderImage() {
        if (znetdkMobile.ajax.requestInProgress === 1) {
            $('#' + znetdkMobile.ajax.loaderElementId).remove();
            $("#zdkmobile-ajax-loading-overlay").remove();
        }
        znetdkMobile.ajax.requestInProgress -= 1;
    }
};

/**
 * Load a remote view and add it to the DOM of the HTML page
 * @param {String} viewName Name of the view to load
 * @param {jQuery} targetHtmlElement The HTML element as jQuery object that will
 * contain the view once loaded
 * @param {function} callback An optional function to call back once the view
 * had been added at the end of the target HTML container
 * @returns {jqXHR|Boolean} The jQuery object returned by the jQuery $.ajax
 * method or false if the view name is not specified and if the target
 * HTML element is not a jQuery element.
 */
znetdkMobile.ajax.loadView = function (viewName, targetHtmlElement, callback) {
    var requestOptions = {
        control: viewName,
        action: 'show',
        htmlTarget: targetHtmlElement
    };
    if (typeof callback === 'function') {
        requestOptions.callback = callback;
    }
    if (targetHtmlElement instanceof jQuery && typeof viewName === 'string') {
        return znetdkMobile.ajax.request(requestOptions);
    }
    return false;
};

/**
 * Execute the last Ajax requests that were queued waiting for authentication
 */
znetdkMobile.ajax.requestFromQueue = function () {
    var ajaxObject = znetdkMobile.ajax,
            queueSize = ajaxObject.requestContext.length;
    for (var index = 0; index < queueSize; index++) {
        ajaxObject.request(ajaxObject.requestContext[index]);
    }
    ajaxObject.requestContext.splice(0, queueSize);
};

/*
 * Initialize the 'controller' and 'action' properties of the specified JS object
 * from the 'data-zdk-load', 'data-zdk-submit' or 'data-zdk-autocomplete'
 * attribute of its 'element' property.
 * See as an example of object to initialize the 'znetdkMobile.form' or the
 * 'znetdkMobile.list' object.
 * @param {String} type The type of controller action to initialize among the
 * 'load', 'submit' and 'autocomplete' choices.
 * @param {Object} object The JS object having the 'action', 'controller' and
 * 'element' properties.
 * @returns {Boolean} Object true when initialization succeeds, false otherwise.
 */
znetdkMobile.ajax.initControllerActionProperties = function (type, object) {
    if (type !== 'load' && type !== 'submit' && type !== 'autocomplete') {
        znetdkMobile.log.error('The type of controller action is invalid!');
        return false;
    }
    if (object === null || typeof object !== 'object') {
        znetdkMobile.log.error('Not a JS object!');
        return false;
    }
    if (!object.hasOwnProperty('element')) {
        znetdkMobile.log.error("'element' property is missing!", object);
        return false;
    }
    if (object.element instanceof jQuery === false) {
        znetdkMobile.log.error('Not a jQuery element!', object.element);
        return false;
    }
    var actionAttrib = object.element.attr('data-zdk-' + type);
    if (typeof actionAttrib === 'string') {
        var actionArray = actionAttrib.split(":");
        if (!object.hasOwnProperty('remoteActions')) {
            object.remoteActions = {};
        }
        object.remoteActions[type] = {
            controller: actionArray[0],
            action: actionArray[1]
        };
        return true;
    } else {
        return false;
    }
};

//************************ BROWSER PUBLIC METHODS **************************

/**
 * Set document title according to the vertical menu active item
 * @param {String} label the label to display as title of the page
 */
znetdkMobile.browser.addLabelToTitle = function (label) {
    var separator = " | ";
    var newTitle = document.title;
    var sepIndex = newTitle.indexOf(separator);
    if (sepIndex === -1) {
        newTitle = label === null || label === "" ? newTitle : label + separator + newTitle;
    } else {
        newTitle = label === null || label === "" ? newTitle.substring(sepIndex + separator.length) : label + separator + newTitle.substring(sepIndex + separator.length);
    }
    document.title = newTitle;
};

/**
 * Returns the path that prefixes the local storage key to avoid the store
 * values to be overwritten when several applications are installed on the same
 * internet domain name.
 * @returns {String} The local storage key path prefix
 */
znetdkMobile.browser.getLocalDataKeyPath = function() {
    var params = znetdkMobile.ajax.getParamsFromAjaxURL(),
            url = params.url.replace('index.php', '');
    if (params.hasOwnProperty('paramName') && params.paramName === 'appl'
            && params.hasOwnProperty('paramValue')) {
        url += params.paramValue + '/';
    }
    return url;
};

/**
 * Stores in the web browser local storage the specified value
 * @param {String} storageKey Identifier of the value stored locally
 * @param {String} value The value to store
 * @returns {Boolean} true if the storage succeeded, false otherwise.
 */
znetdkMobile.browser.storeLocalData = function (storageKey, value) {
    var path = this.getLocalDataKeyPath();
    try {
        localStorage.setItem(path + storageKey, value);
        return true;
    } catch (e) {
        znetdkMobile.log.error('local storage not supported by the browser!');
        return false;
    }
};

/**
 * Retrieves the value stored in the web browser local storage for the
 * specified key
 * @param {String} storageKey Identifier of the value stored locally
 * @returns {DOMString|Boolean} The value found or false if the value does
 * not exist.
 */
znetdkMobile.browser.readLocalData = function (storageKey) {
    var path = this.getLocalDataKeyPath();
    try {
        var storedValue = localStorage.getItem(path + storageKey);
        if (storedValue) {
            return storedValue;
        } else {
            return false;
        }
    } catch (e) {
        znetdkMobile.log.error('local storage not supported by the browser!');
        return false;
    }
};

/**
 * Removes the data matching the specified key from the web browser local storage
 * @param {String} storageKey Identifier of the value stored locally
 */
znetdkMobile.browser.removeLocalData = function (storageKey) {
    var path = this.getLocalDataKeyPath();
    try {
        localStorage.removeItem(path + storageKey);
    } catch (e) {
        znetdkMobile.log.error('local storage not supported by the browser!');
    }
};

/**
 * Declare an handler for the 'resize' events and then adjust the UI when the
 * viewport is resized or when its orientation changes.
 */
znetdkMobile.browser.events.handleViewportResize = function () {
    $(window).on('resize.znetdkmobile', function () {
        /* The page content top spacing is adjusted */
        znetdkMobile.content.setTopSpacing();
        /* The footer position is positioned according to the view content */
        znetdkMobile.footer.adjustPosition();
        /* The action buttons are positioned according to the footer height */
        znetdkMobile.action.adjustPosition();
    });
};

/**
 * Declare an handler for the 'beforeunload' events.
 * A confirmation message is displayed when the browser is closed and when
 * the browser tab containing the application is closed.
 * This confirmation message is not displayed when :
 * - the application is configured to be reloaded each time a new view is
 *   displayed,
 * - the login form is displayed
 * - the 'confirmationOnApplicationClose' property is set to false.
 */
znetdkMobile.browser.events.handleBeforeUnload = function () {
    if (znetdkMobile.browser.confirmationOnApplicationClose
            && !znetdkMobile.navigation.isPageToBeReloaded()
            && !znetdkMobile.authentication.isRequired()) {
        $(window).on('beforeunload.znetdkmobile', function (event) {
            if (!isMailOrTelAnchor()) {
                event.preventDefault();
                event.returnValue = 'Confirmation...';
                return "Confirmation...";
            }
        }).on('unload.znetdkmobile', function (event) {
            if (!isMailOrTelAnchor()) {
                event.preventDefault();
                event.returnValue = 'Confirmation...';
                return "Confirmation...";
            }
        });
    }
    function isMailOrTelAnchor() {
        return document.activeElement.tagName === 'A'
            && (document.activeElement.href.indexOf('mailto:') === 0
            || document.activeElement.href.indexOf('tel:') === 0);
    }
};

znetdkMobile.browser.events.detachBeforeUnload = function () {
    $(window).off('beforeunload.znetdkmobile').off('unload.znetdkmobile');
};

//*************************** HEADER PUBLIC METHODS ****************************

/**
 * Get the header element containing the connected user buttons
 * @returns {jQuery} The element as a jQuery object
 */
znetdkMobile.header.getConnectionArea = function () {
    return $(this.connectionAreaId);
};

/**
 * Get the login name of the connected user
 * @returns {string} The login name
 */
znetdkMobile.header.getConnectedUserLogin = function () {
    return this.getConnectionArea().data('zdk-login');
};

/**
 * Get the connected user name
 * @returns {string} The user name
 */
znetdkMobile.header.getConnectedUserName = function () {
    return this.getConnectionArea().data('zdk-username');
};

/**
 * Get the connected user email
 * @returns {string} The user email
 */
znetdkMobile.header.getConnectedUserMail = function () {
    return this.getConnectionArea().data('zdk-usermail');
};

/**
 * Get the height of the header
 * @returns {integer} Height in pixels
 */
znetdkMobile.header.getHeight = function () {
    return $(this.headerId).height();
};

/**
 * Get the menu button as a jQuery element
 * @returns {jQuery} Button element
 */
znetdkMobile.header.getMenuButton = function () {
    return $(this.menuButtonId);
};

/**
 * Hide the header connection area
 */
znetdkMobile.header.hideConnectionArea = function () {
    $(this.connectionAreaId).addClass('w3-hide');
    $(this.headerId).find('.banner-title-small').addClass('no-connection');
};

/**
 * Show the header connection area
 */
znetdkMobile.header.showConnectionArea = function () {
    $(this.connectionAreaId).removeClass('w3-hide');
    $(this.headerId).find('.banner-title-small').removeClass('no-connection');
};

/**
 * Is header visible ?
 * See znetdkMobile.header.events.handleHideHeaderOnScroll
 * @returns {Boolean} Returns true if visible, false otherwise.
 */
znetdkMobile.header.isVisible = function () {
    return $(znetdkMobile.header.headerId).css('top') === '0px';
};

/**
 * Hides automatically the page header when the page is scrolled down.
 * See znetdkMobile.header.events.handleHideHeaderOnScroll.
 * @param {Boolean|undefined} isEnabled Enabled by default; if set to false, the
 * page header is no longer hidden on scroll.
 */
znetdkMobile.header.autoHideOnScroll = function (isEnabled) {
    this.events.handleHideHeaderOnScroll(isEnabled);
};

/**
 * Display the user panel when clicking on the header profile button
 */
znetdkMobile.header.events.handleProfileButtonClick = function () {
    $('#zdk-profile').on('click.znetdkMobile_header', function (event) {
        znetdkMobile.authentication.showUserPanel();
        event.preventDefault();
    });
};

/**
 * Disconnect the user when clicking on the header logout button
 */
znetdkMobile.header.events.handleLogoutButtonClick = function () {
    $('#zdk-logout').on('click.znetdkMobile_header', function (event) {
        znetdkMobile.authentication.disconnect();
        event.preventDefault();
    });
};

/**
 * Hide the page header on scroll
 * @param {Boolean|undefined} isEnabled When set to false, the page header is
 * no longer hidden on scroll.
 */
znetdkMobile.header.events.handleHideHeaderOnScroll = function(isEnabled) {
    $(window).off('scroll.znetdkMobile_header');
    if (isEnabled === false) {
        znetdkMobile.header.autoHideOnScrollEnabled = false;
        return;
    }
    znetdkMobile.header.autoHideOnScrollEnabled = true;
    $(window).on('scroll.znetdkMobile_header', function() {
        if (document.body.scrollTop > 40 || document.documentElement.scrollTop > 40) {
            znetdkMobile.content.setTopSpacingToTargetAnchor();
            $(znetdkMobile.header.headerId).css('top', znetdkMobile.header.getHeight()*-1);
        } else {
            $(znetdkMobile.header.headerId).css('top', 0);
        }
    });
};

//************************** CONTENT PUBLIC METHODS ****************************

/**
 * Get the view container of the application
 * @returns {jQuery} The container element as jQuery element
 */
znetdkMobile.content.getContainer = function () {
    return $(this.containerId);
};

/**
 * Get the default container of the application
 * @returns {jQuery} The default container element as jQuery element
 */
znetdkMobile.content.getDefaultContainer = function () {
    return $(this.defaultContainerId);
};

/**
 * Display the view container of the application
 */
znetdkMobile.content.showContainer = function () {
    this.getContainer().parent().removeClass('w3-hide');
};

/**
 * Get the ID of the preloaded view into the main application page
 * @returns {String} The identifier of the view or null if no preloaded view
 * exists
 */
znetdkMobile.content.getPreloadedViewId = function () {
    var view = this.getContainer().find('.zdk-filled').eq(0);
    return this.getViewId(view);
};

/**
 * Get the ID of the specified view element
 * @param {jQuery} viewElement The view element
 * @returns {string|null} The view ID
 */
znetdkMobile.content.getViewId = function (viewElement) {
    if (viewElement.length === 1) {
        var splittedID = (viewElement.attr('id')).split("-");
        return splittedID[1];
    }
    return null;
};

/**
 * Display the specified view and scroll its content to the position of the
 * specified anchor.
 * @param {string} viewID The ID of the view
 * @param {string} anchor The HTML ID of the anchor into the view
 */
znetdkMobile.content.displayView = function (viewID, anchor) {
    $('body').trigger('displayview', [viewID, anchor]);
};

/**
 * Reload the specified view and scroll its content to the position of the
 * specified anchor.
 * @param {string} viewID The ID of the view
 * @param {string} anchor The HTML ID of the anchor into the view
 */
znetdkMobile.content.reloadView = function (viewID, anchor) {
    $('body').trigger('displayview', [viewID, anchor, true]);
};


/**
 * Indicates whether the default container is hidden or not
 * @returns {boolean} Value true if hidden
 */
znetdkMobile.content.isDefaultContainerHidden = function () {
    return this.getDefaultContainer().hasClass('ui-helper-hidden');
};

/**
 * Set the top spacing of the content (css padding) to ensure its visibility
 * according to the header height.
 * In addition, a style element is added to the page for adjusting the scroll
 * bar position according to the header height when going to an anchor, except
 * if the page header is hidden on scroll.
 */
znetdkMobile.content.setTopSpacing = function () {
    var headerHeight = znetdkMobile.header.getHeight();
    this.getContainer().css('padding-top', headerHeight);
    this.setTopSpacingToTargetAnchor();
};

/**
 * Set top spacing to the target anchor to ensure its visibility
 * according to the header height except if the page header is hidden on scroll.
 */
znetdkMobile.content.setTopSpacingToTargetAnchor = function () {
    var headerHeight = znetdkMobile.header.getHeight();
    if ($('#zdk-anchor-adjust').length > 0) {
        $('#zdk-anchor-adjust').remove();
    }
    if (znetdkMobile.header.autoHideOnScrollEnabled === false) {
        $('body').append(
                '<style id="zdk-anchor-adjust">'
                + ':target {'
                + 'padding-top:' + headerHeight + 'px;'
                + 'margin-top:-' + headerHeight + 'px;'
                + 'display: inline-block;}'
                + '</style>'
                );
    }
};

/**
 * Display the specified HTML text within the view content
 * @param {string} html The HTML text
 */
znetdkMobile.content.setHtml = function (html) {
    this.getContainer().empty().html(html);
};

/**
 * Get the element of the view currently displayed
 * @returns {jQuery} The view as jQuery Element
 */
znetdkMobile.content.getDisplayedView = function () {
    return this.getContainer().find('.zdk-view:visible');
};

/**
 * Get the ID of the view currently displayed
 * @returns {string} The ID of the displayed view
 */
znetdkMobile.content.getDisplayedViewId = function () {
    var view = this.getDisplayedView();
    return this.getViewId(view);
};

/**
 * Get the view ID from a child element
 * @param {jQuery} childElement The element within the view
 * @returns {String} The identifier of the view
 */
znetdkMobile.content.getParentViewId = function (childElement) {
    if (childElement instanceof jQuery === false) {
        znetdkMobile.log.error('The specified child element is not jQuery object!');
        return null;
    }
    var viewElement = childElement.closest('.zdk-view');
    return this.getViewId(viewElement);
};

/**
 * Indicate whether the specified view exists into the DOM
 * @param {String} viewId ID of the view
 * @returns {Boolean} Value true if the view already exists
 */
znetdkMobile.content.doesViewExistInDom = function (viewId) {
    var view = this.getContainer().find('#znetdk-' + viewId + '-view');
    return view.length === 1;
};

/**
 * Scroll the page to show the specified anchor.
 * If the anchor is not specified, the anchor is read from the URL if it is set
 * @param {String} anchor HTML ID of the anchor
 * @returns {Boolean} Value true when anchor exists and is displayed
 */
znetdkMobile.content.goToAnchor = function (anchor) {
    if (typeof anchor === 'undefined') {// No anchor specified
        anchor = location.hash; // Read from URL
        if (anchor.length === 0) { // No anchor in URL
            return false;
        }
    } else {
        anchor = '#' + anchor;
    }
    if ($(anchor).length !== 1) {
        return false; // anchor element not found
    }
    if (znetdkMobile.navigation.isPageToBeReloaded()) {
        var tempLink = $('<a href="' + anchor + '"></a>').appendTo('body');
        tempLink[0].click();
        tempLink.remove();
    } else {
        var anchorPosition = $(anchor).offset().top,
                headerHeight = znetdkMobile.header.getHeight();
        $(window).scrollTop(anchorPosition - headerHeight);
    }
    return true;
};

//*************************** FOOTER PUBLIC METHODS ****************************

/**
 * Hide the footer
 */
znetdkMobile.footer.hide = function () {
    $(this.footerId).addClass('w3-hide');
};

/**
 * Show the footer
 */
znetdkMobile.footer.show = function () {
    $(this.footerId).removeClass('w3-hide');
};

/**
 * Display the footer at the bottom of the viewport when the page
 * content height is lower than the the viewport height
 */
znetdkMobile.footer.adjustPosition = function () {
    this.hide();
    var footer = $(this.footerId),
            viewPortHeight = $(window).height(),
            bodyHeight = $('body').height(),
            footerHeight = this.getHeight(),
            pageHeight = bodyHeight + footerHeight;
    if (pageHeight > viewPortHeight) {
        footer.css({'position': 'unset', 'bottom': 'unset'});
        footer.find('.adjust-centering').css('margin-right', 'unset');
    } else {
        footer.css({'position': 'fixed', 'bottom': '0'});
        footer.find('.adjust-centering').css('margin-right',
                znetdkMobile.navigation.getVerticalMenuWidth());
    }
    this.show();
};

/**
 * Get the footer height
 * @returns {float} The height of the footer in pixels
 */
znetdkMobile.footer.getHeight = function () {
    var footerElement = $(this.footerId),
            footerHeight = footerElement.outerHeight(true);
    if (footerHeight === 0) { // The footer hidden so the height returned is zero
        footerElement.css('visibility', 'hidden');
        this.show();
        footerHeight = footerElement.outerHeight(true);
        this.hide();
        footerElement.css('visibility', 'visible');
    }
    return footerHeight;
};


//*************************** MESSAGE PUBLIC METHODS ***************************

/**
 * Get the message container
 * @returns {jQuery} The container element as jQuery element
 */
znetdkMobile.messages.getContainer = function () {
    return $(this.containerId);
};

/**
 * Instanciate the message to display within the message container
 * @param {jQuery} messageElement The DIV element of the message as jQuery element
 * @returns {Object|null} The message as an Object or null if instanciation failed
 */
znetdkMobile.messages.make = function (messageElement) {
    if (messageElement instanceof jQuery === false) {
        znetdkMobile.log.error('Not a jQuery element!');
        return null;
    }
    if (messageElement.length !== 1) {
        znetdkMobile.log.error('Message element is empty!');
        return null;
    }
    var messageObject = Object.create(this);
    messageObject.element = messageElement;
    return messageObject;
};

/**
 * Display the specified message
 * @param {String} severity The severity of the message: 'info', 'warn', 'error'
 * or 'critical'
 * @param {String} summary Title of the message
 * @param {String} detail Text of the message
 * @param {Boolean} autoHide If set to true or undefined, the message is
 * automatically hidden after the time set in seconds for the
 * 'autoCloseDuration' property
 */
znetdkMobile.messages.add = function (severity, summary, detail, autoHide) {
    var message = $('<div class="w3-panel w3-card w3-animate-top w3-display-container '
            + this.colors[severity] + '">'
            + '<span class="close w3-button w3-large w3-display-topright"><i class="fa fa-times"></i></span>'
            + '<h3><i class="fa ' + this.icons[severity] + '"></i>&nbsp;' + summary + '</h3><p>' + detail + '</p></div>');
    var addedMessage = message.appendTo(this.getContainer());
    if (severity !== 'critical' && (typeof autoHide === 'undefined' || autoHide === true)) {
        window.setTimeout(function () {
            znetdkMobile.messages.make(addedMessage).remove();
        }, this.autoCloseDuration);
    }
    znetdkMobile.content.setTopSpacing();
    znetdkMobile.footer.adjustPosition();
};

/**
 * Add multiples messages into the message area
 * @param {array} messages The messages to add to the message container
 * @returns {Boolean} Value true when succeeded
 */
znetdkMobile.messages.addMulti = function (messages) {
    if (!Array.isArray(messages)) {
        znetdkMobile.log.error('Not an array!');
        return false;
    }
    var $this = this;
    messages.forEach(function (message) {
        if (Array.isArray(message)) {
            $this.add.apply($this, message);
        }
    });
    return true;
};

/**
 * Remove the message from the message container
 * @returns {Boolean} Value true when succeeded
 */
znetdkMobile.messages.remove = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Not a jQuery element!');
        return false;
    }
    this.element.remove();
    znetdkMobile.content.setTopSpacing();
    znetdkMobile.footer.adjustPosition();
    return true;
};

/**
 * Remove all the messages from the message container
 */
znetdkMobile.messages.removeAll = function () {
    this.getContainer().empty();
    znetdkMobile.content.setTopSpacing();
    znetdkMobile.footer.adjustPosition();
};

/**
 * Display the message as a snackbar message
 * @param {String} text The text of the message
 * @param {undefined|Boolean} isWarning When set to true, the snackbar message
 * is displayed as a warning message
 * @param {undefined|jQuery} parentContainer Optional jQuery element containing
 * the snackbar message (useful for display over a modal dialog).
 */
znetdkMobile.messages.showSnackbar = function (text, isWarning, parentContainer) {
    var color = isWarning === true ? 'w3-yellow' : 'w3-green',
            icon = isWarning === true ? 'fa-exclamation-triangle' : 'fa-check-circle',
            message = $('<div class="w3-panel w3-card w3-animate-bottom ' + color + '">'
                    + '<p class="w3-center"><i class="fa ' + icon + '"></i>&nbsp;' + text + '</p></div>');
    message.css({
        position: 'fixed',
        bottom: '0px',
        left: '25%',
        width: '50%'
    });
    var addedMessage = message.appendTo(typeof parentContainer === 'undefined'
        ? this.getContainer() : parentContainer);
    window.setTimeout(function () {
        addedMessage.remove();
    }, 2500);
};

/**
 * Display a notification message
 * @param {String} title Title displayed into the title bar
 * @param {String} message The notification message
 * @param {String} buttonLabel The label of the OK button ('Ok' if undefined)
 * @param {function} callback The function to call back when the OK button is
 * pressed
 */
znetdkMobile.messages.notify = function (title, message, buttonLabel, callback) {
    var modal = $('#zdk-notification-modal');
    modal.find('.title').html(title);
    modal.find('.message').html(message);
    if (typeof buttonLabel === 'string') {
        modal.find('button').html(buttonLabel);
    }
    modal.find('button').one('click.znetdkMobile_messages', function () {
        modal.hide();
        if (typeof callback === 'function') {
            callback();
        }
    });
    modal.show();
};

/**
 * Display a message for confirmation purpose
 * @param {String} title Title displayed into the title bar
 * @param {String} question Text of the question asked to the user
 * @param {Object} buttons Labels of the YES and NO buttons ('Yes' and 'No' by
 * default)
 * @param {function} callback The function to call back when the YES or NO button
 * is pressed. The parameter passed to the function is a Boolean variable set to
 * true when the YES button is pressed
 */
znetdkMobile.messages.ask = function (title, question, buttons, callback) {
    var modal = $('#zdk-confirmation-modal'),
            yesLabel, noLabel;
    modal.find('.title').html(title);
    modal.find('.message').html(question === null
        ? modal.find('.message').data('default-msg') : question);
    if (buttons !== null && typeof buttons === 'object'
            && buttons.hasOwnProperty('yes') && buttons.hasOwnProperty('no')) {
        yesLabel = buttons.yes;
        noLabel = buttons.no;
    } else {
        yesLabel = modal.find('button.yes span.label').data('default-label');
        noLabel = modal.find('button.no span.label').data('default-label');
    }
    modal.find('button.yes span.label').html(yesLabel);
    modal.find('button.no span.label').html(noLabel);
    modal.find('button').off('click.znetdkMobile_messages');
    modal.find('button').one('click.znetdkMobile_messages', function () {
        modal.hide();
        if (typeof callback === 'function') {
            callback($(this).hasClass('yes'));
        }
    });
    modal.show();
};

/**
 * Close a message when its close button is pressed
 */
znetdkMobile.messages.events.handleAllClose = function () {
    $('#zdk-messages').on('click.znetdkmobile_messages', 'span.close', function () {
        znetdkMobile.messages.make($(this).parent()).remove();
    });
};

//***************************** LOG PUBLIC METHODS *****************************

/**
 * Display into the Web browser console the specified message as a warning
 * message
 * @param {String} message Text of the message
 * @param {Object} objectToLog Object to log into the console
 * @param {String} severity Value 'error' when it is an error, otherwise is
 *  a warning message
 */
znetdkMobile.log.warn = function (message, objectToLog, severity) {
    var consoleMethod;
    if (severity === 'error') {
        consoleMethod = console.error;
    } else {
        consoleMethod = console.warn;
    }
    if (typeof objectToLog === 'object') {
        consoleMethod('[znetdkMobile]', message, ', Traced object:', objectToLog);
    } else {
        consoleMethod('[znetdkMobile]', message);
    }
};

/**
 * Display into the Web browser console the specified message as an error
 * message
 * @param {String} message Text of the message
 * @param {Object} objectToLog Object to log into the console
 */
znetdkMobile.log.error = function (message, objectToLog) {
    this.warn(message, objectToLog, 'error');
};

//*********************** AUTHENTICATION PUBLIC METHODS ************************

/**
 * Indicates whether authentication is enabled or not for the application
 * @returns {Boolean} Value true when authentication is enabled
 */
znetdkMobile.authentication.isEnabled = function () {
    var loginName = znetdkMobile.header.getConnectedUserLogin();
    return loginName !== '';
};

/**
 * Indicates whether the user must authenticated or not
 * @returns {Boolean} Value true when user must be authenticated
 */
znetdkMobile.authentication.isRequired = function () {
    return (znetdkMobile.navigation.getMenuDefinition().length === 0
            && znetdkMobile.content.isDefaultContainerHidden());
};

/**
 * Show the change password modal dialog
 * @param {String} loginName The user login name
 * @param {String} errorMessage In option an error message to display into the
 *  change password form
 */
znetdkMobile.authentication.changePassword = function (loginName, errorMessage) {
    var $this = this,
            isUserConnected = typeof errorMessage === 'undefined',
            modal = znetdkMobile.modal.make($(this.changePasswordFormId)),
            innerForm = modal.getInnerForm();
    innerForm.init({login_name: loginName});
    modal.open(
            function (response) { // On submit
                if (!isUserConnected && response.success === true) {
                    location.reload(); // The page is reloaded
                }
            },
            function () { // On close
                if (!isUserConnected) {
                    $this.cancelLogin(modal);
                    return false; // Modal not closed by default event handler
                }
            }
    );
    if (typeof errorMessage === 'string') {
        innerForm.showError(errorMessage);
    }
};

/**
 * Display the user panel showing his user name and email address and
 * allowing to change his password
 */
znetdkMobile.authentication.showUserPanel = function () {
    var $this = this,
            modalElement = $(this.connectedUserPanelId),
            modal = znetdkMobile.modal.make(modalElement),
            userName = znetdkMobile.header.getConnectedUserName(),
            email = znetdkMobile.header.getConnectedUserMail();
    modalElement.find('h3.username').text(userName);
    modalElement.find('p.usermail').text(email);
    modal.open();
    _handleChangePasswordButtonClick();
    function _handleChangePasswordButtonClick() {
        var eventName = 'click.znetdkmobile_authentication',
                button = modalElement.find('button.changepwd');
        button.off(eventName);
        button.on(eventName, function () {
            var login = znetdkMobile.header.getConnectedUserLogin();
            modal.close();
            $this.changePassword(login);
        });
    }
};

/**
 * Display the login form in a modal dialog
 * @param {Boolean} renewCredentials When set to true, the login name is
 * pre-filled and the keyboard focus is set on the password field
 */
znetdkMobile.authentication.showLoginForm = function (renewCredentials) {
    var $this = this,
            modal = znetdkMobile.modal.make($(this.loginFormId)),
            focusedField;
    if (renewCredentials === true) {
        // The login name is pre-filled
        modal.getInnerForm().setInputValue('login_name',
                znetdkMobile.header.getConnectedUserLogin());
        // The focus is set on the password field
        focusedField = 'password';
    }
    handleRememberMeClick();
    if (initRememberMeState()) {
        // Forgot password link only displayed on trusted terminal
        handleForgotPasswordClick();
    }
    modal.open(
            function (response) { // On submit
                if (response.success === true) {
                    if (renewCredentials === true) {
                        modal.close();
                        znetdkMobile.ajax.requestFromQueue();
                    } else {
                        memorizeRememberMeState();
                        location.reload(); // The page is reloaded
                    }
                }
                if (response.newpasswordrequired) { // Change password required
                    modal.close();
                    $this.changePassword(modal.getInnerForm().getInputValue('login_name'),
                            response.msg);
                    // The error message must not be displayed
                    // by the form submit event default handler.
                    return false;
                } else if (response.toomuchattempts) { // User account disabled
                    // The login button is disabled
                    modal.getInnerForm(true).find('button[type=submit]').prop('disabled', true);
                }
            },
            function () { // On close
                $this.cancelLogin(modal);
                // The modal dialog is closed by the cancelLogin method.
                // So it must not be closed by the default 'close' event handler.
                return false;
            },
            focusedField // Form Field with focus
            );
    function handleRememberMeClick() {
        $('#zdk-login-modal-remember-me').off('change.znetdkmobile')
                .on('change.znetdkmobile', function () {
            var accessElement = $(this).closest('form').find('input[name=access]'),
                accessValue = $(this).is(':checked') ? 'private' : 'public';
            accessElement.val(accessValue);
        });
    }
    function initRememberMeState() {
        var accessValue = znetdkMobile.browser.readLocalData($this.rememberMeLocalStorageKey),
                rememberMeElement = $('#zdk-login-modal-remember-me');
        if (rememberMeElement.length === 1 && accessValue !== false) {
            var accessElement = modal.getInnerForm(true).find('input[name=access]');
            accessElement.val(accessValue);
            rememberMeElement.prop('checked', accessValue === 'private');
        }
        return accessValue !== false;
    }
    function memorizeRememberMeState() {
        var accessValue = modal.getInnerForm().getInputValue('access');
        znetdkMobile.browser.storeLocalData($this.rememberMeLocalStorageKey, accessValue);
    }
    function handleForgotPasswordClick() {
        modal.getInnerForm(true).find('.zdk-forgot-pwd').off('click.znetdkmobile')
                .one('click.znetdkmobile', function(event) {
            znetdkMobile.ajax.loadView('forgotpassword', $('body'), function() {
                modal.close();
                var resetPwdModal = znetdkMobile.modal.make('#mzdk_forgot_password_dialog');
                resetPwdModal.open(onCloseModal, onCloseModal);
                function onCloseModal() {
                    $this.cancelLogin(modal);
                }
            });
            event.preventDefault();
        }).removeClass('w3-hide');
    }
};

/**
 * Disconnect the connected user
 */
znetdkMobile.authentication.disconnect = function () {
    var $this = this;
    znetdkMobile.ajax.request({
        control: 'security',
        action: 'logout',
        callback: function (response) {
            $this.setNoConnectionState(response.msg);
            // No longer confirmation message on application reload or close
            znetdkMobile.browser.events.detachBeforeUnload();
        }
    });
};

/**
 * Cancel le login process and close the login form
 * @param {Object} modal The modal dialog object of the login form
 */
znetdkMobile.authentication.cancelLogin = function (modal) {
    var $this = this;
    znetdkMobile.ajax.request({
        control: 'security',
        action: 'cancellogin',
        callback: function (response) {
            $this.setNoConnectionState(response.msg);
            modal.close();
        }
    });
};

/**
 * Change the display of the application in the case of a user who is no longer
 * connected
 * @param {String} message Message to display in the view container to notify
 * the user that is no longer connected to the application
 */
znetdkMobile.authentication.setNoConnectionState = function (message) {
    znetdkMobile.header.hideConnectionArea();
    /* Navigation menus are hidden */
    znetdkMobile.navigation.setNoNavigation();
    /* The page content top spacing is adjusted */
    znetdkMobile.content.setTopSpacing();
    /* The Action buttons are hidden */
    znetdkMobile.action.hide();
    /* Display of the disconnection confirmation message */
    znetdkMobile.content.setHtml(message);
    /* The page content is displayed if hidden */
    znetdkMobile.content.showContainer();
    /* Footer position is reset */
    znetdkMobile.footer.adjustPosition();
};


//************************* NAVIGATION PUBLIC METHODS **************************

/**
 * Get the menu definition of the application as a jQuery element
 * @returns {jQuery} The menu element
 */
znetdkMobile.navigation.getMenuDefinition = function () {
    return $(this.menuDefinitionId);
};

/**
 * Get the vertical menu element as a jQuery element
 * @returns {jQuery} The vertical menu element
 */
znetdkMobile.navigation.getVerticalMenu = function () {
    return $(this.verticalMenuId);
};

/**
 * Get the horizontal menu element as a jQuery element
 * @returns {jQuery} The horizontal menu
 */
znetdkMobile.navigation.getHorizontalMenu = function () {
    return $(this.horizontalMenuId);
};

/**
 * Get the vertical menu width
 * @returns {float} The vertical menu width in pixels
 */
znetdkMobile.navigation.getVerticalMenuWidth = function () {
    return this.getMenuDefinition().width();
};

/**
 * Build the navigation menu of the application from its menu definition
 * loaded into the main HTML page
 * @returns {Boolean} Value true when menu building succeeded. Returns false
 * when the application is configured with CFG_VIEW_PAGE_RELOAD = TRUE (see
 * config.php script) and a HTTP 404 Error occured (view unknown).
 */
znetdkMobile.navigation.build = function () {
    var $this = this;
    // Extra initialization
    _hideSubItems();
    _addW3cssClassesToMenu();
    _displayMenuIcons();
    _showMenu(); // Menu is initialy hidden for preventing FOUC effect
    znetdkMobile.content.showContainer();// The view container is displayed (hidden when main page is loaded)

    // Display the initial view once the main page is loaded according to the
    // CFG_VIEW_PAGE_RELOAD parameter value:
    // If FALSE, the view matching the first menu item is displayed
    // If TRUE, the embedded view into the main page is shown
    if (_displayInitialView() === false) { //CFG_VIEW_PAGE_RELOAD === TRUE && HTTP Error 404
        // The page content top spacing is adjusted
        znetdkMobile.content.setTopSpacing();
        // The footer position is adjusted
        znetdkMobile.footer.adjustPosition();
        return false;
    }
    // Bind events
    _bindEvents();

    return true;

    // Internal private functions
    function _hideSubItems() {
        $this.getMenuDefinition().find('li.has-sub > ul').hide();
    }
    function _addW3cssClassesToMenu() {
        $this.getMenuDefinition().find('a').addClass('w3-bar-item w3-button w3-hover-theme');
    }
    function _displayMenuIcons() {
        $this.getMenuDefinition().find('li > a').each(function () {
            var icon = $(this).data('icon');
            if (typeof icon === 'string' && icon.substr(0, 3) === 'fa-') {
                $(this).prepend('<i class="fa ' + icon + ' fa-lg"></i>');
            }
        });
    }
    function _showMenu() {
        $this.getMenuDefinition().css('display', 'block');
    }
    function _bindEvents() {
        // Click on the vertical menu button
        znetdkMobile.header.getMenuButton().on('click.znetdkmobile', function (event) {
            $this.openVerticalMenu();
            event.preventDefault();
        });
        // Click on the vertical menu close button
        $this.getVerticalMenu().find('button.close').on('click.znetdkmobile', function () {
            $this.closeVerticalMenu();
        });
        // Click on the company logo
        $($this.companyLogoId).on('click.znetdkmobile', function (event) {
            if (!$this.isPageToBeReloaded()) {
                _displayInitialView();
                event.preventDefault();
            }
        });
        // Click on a vertical menu item
        $this.getMenuDefinition().on('click.znetdkmobile_navigation', 'a', function (event) {
            var menuItemId = $(this).next('ul').length === 0
                    ? _getMenuItemId($(this).parent()) // No subitem
                    : _getFirstChildMenuItemId($(this).parent()); // Subitems exist
            _displayView(menuItemId);
            event.preventDefault();
        });
        // Click on a horizontal menu item
        $this.getHorizontalMenu().on('click.znetdkmobile_navigation', 'a', function (event) {
            // Handled only if click on a hyperlink
            var menuItemId = $(this).data('view_id');
            _displayView(menuItemId);
            event.preventDefault();
        });
        // Display view sent event
        $('body').on('displayview.znetdkmobile_navigation', function (event, viewID, anchor, reload) {
            _displayView(viewID, anchor, reload);
        });
    }
    function _displayInitialView() {
        if ($this.getMenuDefinition().length === 0) {
            // Menu is missing (404 HTTP error)
            return false;
        }
        if ($this.isPageToBeReloaded()) {
            // The view already exists into the main page
            _displayPreloadedView();
        } else {
            // The view is loaded by AJAX request
            _displayView();
        }
        function _displayPreloadedView() {
            var innerViewId = znetdkMobile.content.getPreloadedViewId(),
                    item = _getMenuItem(innerViewId);
            if (_initHorizontalMenu(item) === false) {
                return false;
            }
            // The menu items are activated in the vertical and horizontal menus
            _setVerticalMenuItemActive(innerViewId);
            _setHorizontalItemActive(innerViewId);
            // The page content top spacing is adjusted
            znetdkMobile.content.setTopSpacing();
            // The footer position is adjusted
            znetdkMobile.footer.adjustPosition();
            // If an anchor is set into the URL, the page is scroll to the anchor
            znetdkMobile.content.goToAnchor();
        }
    }
    function _displayView(viewID, anchor, reload) {
        var noViewSpecified = viewID === undefined,
                menuItemId = noViewSpecified ? _getFirstChildMenuItemId() : viewID,
                containerID = _getViewContainerID(menuItemId),
                containerElement = $('#' + containerID),
                containerExists = (containerElement.length > 0),
                menuItemElement = _getMenuItem(menuItemId);
        if (menuItemElement === false) {
            znetdkMobile.log.error("The view ID = '" + viewID + "' is unknown!");
            return false;
        }
        if ($this.isPageToBeReloaded()) { /* Page reload */
            // The page is reloaded to be replaced by the one matching the new view ID
            // If an anchor is specified, it is added at the end of the URI
            var uriAnchor = typeof anchor === 'undefined' ? '' : '#' + anchor;
            location.assign(menuItemElement.children('a').first().attr('href') + uriAnchor);
            return;
        }
        if (containerExists && (reload === true
                || containerElement.find('.' + $this.autoReloadViewClass).length === 1)) {
            // The view is removed first
            containerElement.remove();
            containerExists = false;
        }
        if (containerExists) {
            // View container already exists...
            _showView(menuItemId, anchor);
        } else {
            // View container does not yet exist...
            containerElement = $('<div id="' + containerID + '" class="zdk-view"/>');
            _getViewsContainer().prepend(containerElement);
            containerElement.hide();
            znetdkMobile.ajax.loadView(menuItemId, containerElement, function () {
                _showView(menuItemId, anchor);
            });
        }
        function _showView(viewID, anchor) {
            var viewElement = $('#' + _getViewContainerID(viewID)),
                    viewIsVisible = viewElement.is(':visible'),
                    displayedView = _getViewsContainer().children(".zdk-view").filter(":visible");
            $this.events.triggerBeforeViewDisplay(viewID);
            if (!viewIsVisible && displayedView.length >= 1) {
                // The current displayed view is first hidden if it is not yet
                // the view to display
                displayedView.fadeOut(200);
                displayedView.promise().done(function () {
                    showObj();
                });
            } else {
                showObj();
            }
            function showObj() {
                _initHorizontalMenu(_getMenuItem(viewID)); // Horizontal menu initialized
                _setHorizontalItemActive(viewID); // Horizontal menu item set active
                _setVerticalMenuItemActive(viewID); // Vertical menu item set active
                znetdkMobile.browser.addLabelToTitle(// Menu item label displayed as browser tab label
                        _getMenuItemLabel(menuItemId));
                if (!viewIsVisible) {
                    // The view is displayed if it is not already displayed
                    viewElement.fadeIn(200, function () {
                        $this.events.triggerAfterViewDisplay(viewID);
                        znetdkMobile.content.goToAnchor(anchor); // The view is positionned on the anchor
                    });
                } else {
                    $this.events.triggerAfterViewDisplay(viewID);
                    znetdkMobile.content.goToAnchor(anchor); // The view is positionned on the anchor
                }
                function _getMenuItemLabel(viewID) {
                    return _getMenuItem(viewID).children("a").text();
                }
            }
        }
    }
    function _initHorizontalMenu(verticalMenuItem) {
        if (verticalMenuItem === false) {
            return false;
        }
        if (_isRootMenuItem(verticalMenuItem)) {
            _addItemToHorizontalMenu(verticalMenuItem, true);
        } else {
            _addSubItemsToHorizontalMenu(verticalMenuItem.parent('ul'));
        }
        /* The page content top spacing is adjusted */
        znetdkMobile.content.setTopSpacing();
        return true;
        function _isRootMenuItem(menuElement) {
            var rootItem = _getRootMenuItem(menuElement);
            return rootItem === menuElement;
        }
    }
    function _getRootMenuItem(menuElement) {
        var result = menuElement;
        menuElement.parentsUntil($this.getMenuDefinition()).filter('li').each(function () {
            if ($(this).parentsUntil($this.getMenuDefinition()).filter('li').length === 0) {
                result = $(this);
                return false;
            }
        });
        return result;
    }
    function _getMenuItem(viewID) {
        var menuItem = $this.getMenuDefinition().find('ul > li[id=znetdk-' + viewID + '-menu]');
        return menuItem.length === 1 ? menuItem : false;
    }
    function _getFirstChildMenuItem(parent) {
        if (typeof parent === 'undefined') {
            parent = $this.getMenuDefinition().find('ul > li').first();
        }
        if (parent.length) {
            var child = parent.find('ul>li').first();
            if (child.length) {
                return _getFirstChildMenuItem(child);
            } else {
                return parent;
            }
        } else {
            return false;
        }
    }
    function _getFirstChildMenuItemId(parent) {
        var item = _getFirstChildMenuItem(parent);
        if (item === false) {
            return false;
        }
        return _getMenuItemId(item);
    }
    function _getMenuItemId(menuElement) {
        if (menuElement instanceof jQuery && menuElement.length === 1) {
            var splittedID = (menuElement.attr('id')).split("-"),
                menuItemId = '';
            for (var i=1; i < splittedID.length-1; i++) {
                menuItemId += (menuItemId.length > 0 ? '-' : '') + splittedID[i];
            }
            return menuItemId;
        }
        return null;
    }
    function _setVerticalMenuItemActive(viewID) {
        var menuItem = _getMenuItem(viewID);
        _resetMenuItemActive();
        menuItem.addClass($this.verticalMenuActiveItemClass);
        _getRootMenuItem(menuItem).addClass($this.verticalMenuActiveItemClass);
        function _resetMenuItemActive() {
            $this.getMenuDefinition().find('li.' + $this.verticalMenuActiveItemClass)
                    .removeClass($this.verticalMenuActiveItemClass);
        }
    }
    function _setHorizontalItemActive(viewID) {
        var menuItem = null;
        if (typeof viewID === 'undefined') {
            menuItem = $this.getHorizontalMenu().find('.items a:first');
        } else {
            _resetTabItemActive();
            menuItem = $this.getHorizontalMenu().find('.items a[data-view_id=' + viewID + ']');
            if (menuItem.length === 0) {
                znetdkMobile.log.error("Horizontal tab menu: view ID='" + viewID + "' unknown.");
            }
        }
        menuItem.find('.w3-bottombar')
                .addClass($this.horizontalMenuActiveItemClass)
                .addClass($this.horizontalMenuActiveItemBorderClass)
                .removeClass($this.horizontalMenuItemBorderClass);
        function _resetTabItemActive() {
            $this.getHorizontalMenu().find('.items .w3-bottombar')
                    .removeClass($this.horizontalMenuActiveItemClass)
                    .removeClass($this.horizontalMenuActiveItemBorderClass)
                    .addClass($this.horizontalMenuItemBorderClass);
        }
    }
    function _addItemToHorizontalMenu(menuItem, onlyOne) {
        var tabmenu = $this.getHorizontalMenu(),
                link = menuItem.children('a'),
                icon = link.data('icon'),
                label = link.text(),
                newItem = tabmenu.find('.template > a').clone();
        if (typeof icon === 'undefined') {
            newItem.find('i').remove(); // No icon set
        } else {
            newItem.find('i').addClass(icon); // Icon exists
        }
        newItem.find('span').text(label);
        newItem.attr('data-view_id', _getMenuItemId(menuItem));
        if (onlyOne === true) {
            tabmenu.find('.items').empty();
        }
        tabmenu.find('.items').append(newItem);
    }
    /**
     * Display the tab menu from the clicked side nav menu item
     * @param {jQuery} parentItem Parent UL element of the side nav menu
     * containing the submenu definition.
     */
    function _addSubItemsToHorizontalMenu(parentItem) {
        var tabmenu = $this.getHorizontalMenu();
        tabmenu.find('.items').empty();
        parentItem.find('>li').each(function () {
            _addItemToHorizontalMenu($(this));
        });
    }
    function _getViewContainerID(viewID) {
        return 'znetdk-' + viewID + '-view';
    }
    function _getViewsContainer() {
        var container = znetdkMobile.content.getContainer();
        return container.length ? container : $('body');
    }
}
;

/**
 * Returns true when the application is configured (CFG_VIEW_PAGE_RELOAD = TRUE)
 * to force the main application page to be reloaded each time a new view is
 * displayed.
 * @returns {boolean} true if the main page is to reload for each view displayed,
 * false otherwise.
 */
znetdkMobile.navigation.isPageToBeReloaded = function () {
    return znetdkMobile.navigation.getMenuDefinition().hasClass('zdk-pagereload');
};

/**
 * Hide the vertical and horizontal menus of the application
 */
znetdkMobile.navigation.setNoNavigation = function () {
    $('.w3-main').css('margin-left', '0');
    this.getVerticalMenu().css('visibility', 'hidden');
    this.getHorizontalMenu().css('visibility', 'hidden');
    znetdkMobile.header.getMenuButton().css('visibility', 'hidden');
};

/**
 * Show the vertical menu
 */
znetdkMobile.navigation.openVerticalMenu = function () {
    this.getVerticalMenu().show();
};

/**
 * Hide the vertical menu
 */
znetdkMobile.navigation.closeVerticalMenu = function () {
    this.getVerticalMenu().hide();
};

/**
 * Display on the right of the horizontal menu item label, the count of rows
 * displayed into the view's list
 * @param {integer} rowCount The number of rows
 * @param {String} viewId The ID of the view
 * @returns {Boolean} Value true when succeeded, false when the view ID does not
 * match any view loaded into the view container
 */
znetdkMobile.navigation.addRowCountToHorizontalMenuItem = function (rowCount, viewId) {
    var menuItem = this.getHorizontalMenu().find('.items a[data-view_id=' + viewId + ']');
    if (menuItem.length === 0) {
        znetdkMobile.log.error('Unable to get the horizontal menu item for the displayed view!');
        return false;
    }
    if (menuItem.find('span.row-count').length === 0) {
        menuItem.find('span').last().after('<span class="row-count"/>');
    }
    menuItem.find('span.row-count').text(' (' + rowCount + ')');
    /* The page content top spacing is adjusted */
    znetdkMobile.content.setTopSpacing();
    return true;
};

/**
 * Clear the row count displayed on the right of the horizontal menu item label.
 * @param {String} viewId The ID of the view
 * @returns {Boolean} Value true when succeeded, false when the view ID does not
 * match any view loaded into the view container
 */
znetdkMobile.navigation.clearRowCountFromHorizontalMenuItem = function (viewId) {
    var menuItem = this.getHorizontalMenu().find('.items a[data-view_id=' + viewId + ']');
    if (menuItem.length === 0) {
        znetdkMobile.log.error('Unable to get the horizontal menu item for the displayed view!');
        return false;
    }
    menuItem.find('span.row-count').remove();
    /* The page content top spacing is adjusted */
    znetdkMobile.content.setTopSpacing();
    return true;
};

/**
 * Trigger the 'beforeviewdisplay' event just before displaying the specified
 * view
 * @param {String} viewId The ID of the view
 */
znetdkMobile.navigation.events.triggerBeforeViewDisplay = function (viewId) {
    /* Footer is temporarily hidden */
    znetdkMobile.footer.hide();
    /* Action buttons are temporarily hidden */
    znetdkMobile.action.hide();
    /* The page content top spacing is adjusted */
    znetdkMobile.content.setTopSpacing();
    /* Custom event is triggered */
    $('body').trigger(znetdkMobile.navigation.events.beforeViewDisplayName, [viewId]);
};

/**
 * Trigger the 'afterviewdisplay' event just after displaying the specified
 * view
 * @param {String} viewId The ID of the view
 */
znetdkMobile.navigation.events.triggerAfterViewDisplay = function (viewId) {
    /* The sidebar is hidden after clicking on an item menu and the page scrolled up */
    znetdkMobile.navigation.closeVerticalMenu();
    /* Footer position is adjusted */
    znetdkMobile.footer.adjustPosition();
    /* Action buttons are adjusted */
    znetdkMobile.action.adjustPosition();
    /* Custom event is triggered */
    $('body').trigger(znetdkMobile.navigation.events.afterViewDisplayName, [viewId]);
};


//*************************** MODAL PUBLIC METHODS *****************************

/**
 * Instantiate the modal dialog from the specified element.
 * @param {String|jQuery} modalElementSelector The string selector of the modal
 * (i.e '#my-modal') or a jQuery element of the modal.
 * @returns {modalObject|null} The modal object or null if the specified element
 * is invalid
 */
znetdkMobile.modal.make = function (modalElementSelector) {
    var modalElement = modalElementSelector instanceof jQuery
            ? modalElementSelector : $(modalElementSelector);
    if (modalElement.length !== 1) {
        znetdkMobile.log.error('Modal element is empty or multiple!');
        return null;
    }
    if (!modalElement.hasClass(this.cssClass)) {
        znetdkMobile.log.error('Modal CSS class missing!', modalElement);
        return null;
    }
    var modalObject = Object.create(this);
    modalObject.element = modalElement;
    return modalObject;
};

/**
 * Returns the form of the modal.
 * If more than one form exist, only the first one is returned.
 * @param {Boolean} asJqueryElement If true, the form is return as a jQuery
 *  element. Otherwise, the form is returned as a znetdkMobile object.
 * @param {String} selectorClass Optional CSS class of the form if multiple
 *  forms exist within the modal (for example 'my-form').
 * @returns {znetdkMobile.form|jQuery|Boolean} The form of the modal by default
 * as a znetdkMobile object or as a jQuery element. Returns false if no form
 * exists or if more than one form exist.
 */
znetdkMobile.modal.getInnerForm = function (asJqueryElement, selectorClass) {
    var form = this.element.find('form'
        + (typeof selectorClass === 'string' ? '.' + selectorClass : '')
    );
    if (form.length === 0) {
        return false;
    } else if (form.length > 1) {
        form = form.first();
    }
    return asJqueryElement === true ? form : znetdkMobile.form.make(form);
};

/**
 * Display the modal dialog
 * @param {function} onSubmit The function to callback when clicking on the
 * inner form submit button. If this callback function returns false, the modal
 * dialog is not closed once submit succeeded.
 * @param {function} onClose The function to callback when closing the modal
 * dialog. If this callback function returns false, the modal dialog is not
 * automatically closed.
 * @param {String} focusedInputName The name of the inner form input element
 * for which the keyboard focus is set
 * @returns {Boolean} Value true when it is opened successfully, false otherwise
 */
znetdkMobile.modal.open = function (onSubmit, onClose, focusedInputName) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Modal is not instantiated!');
        return false;
    }
    _registerCloseCallback(this, onClose);
    this.element.show();
    _initForm(this, onSubmit);
    return true;
    // Private functions
    function _registerCloseCallback(modalObject, closeCallback) {
        if (typeof closeCallback !== 'function') {
            return false;
        }
        var eventName = modalObject.events.beforeCloseNane + '.znetdkmobile_modal';
        modalObject.element.off(eventName);
        modalObject.element.on(eventName, function () {
            return closeCallback();
        });
    }
    function _initForm(modalObject, submitCallback) {
        var form = modalObject.getInnerForm(true);
        if (form === false) {
            return false;
        }
        var formObject = znetdkMobile.form.make(form, function (response) {
            var returnedValue = typeof submitCallback === 'function'
                    ? submitCallback(response) : true;
            if (returnedValue !== false && response.success === true
                    && modalObject.closeOnSubmitSuccess === true) {
                modalObject.close();
            }
            return returnedValue;
        });
        if (formObject.doesInputExist(focusedInputName)) {
            formObject.setFocus(focusedInputName);
        } else {
            formObject.setFocusOnFirstInput();
        }
    }

};

/**
 * Close the modal dialog
 * @param {Boolean|undefined} checkDataFormsModified When set to true,
 * check if the inner input form data are modified before closing.
 * If so, a confirmation message is displayed before.
 * @returns {Boolean} Value true if modal is closed, false otherwise
 */
znetdkMobile.modal.close = function (checkDataFormsModified) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Modal is not instantiated!');
        return false;
    }
    var modalElement = this.element;
    _areDataFormsModified(function(){
        modalElement.hide();
    });

    return true;
    function _areDataFormsModified(callback) {
        if (checkDataFormsModified === false || typeof checkDataFormsModified === 'undefined') {
            return callback(); // No checking
        }
        var isModified = false;
        modalElement.find('form[data-zdk-submit]').each(function(){
            var formObject = znetdkMobile.form.make($(this));
            if (formObject.isModified()) {
                isModified = true;
                return false; // Break the loop
            }
        });
        if (!isModified) {
            return callback(); // No data modified into the forms
        }
        znetdkMobile.messages.ask(modalElement.find('button.cancel').text(),
            null, null, function(isYes){
            if (isYes) {
                callback(); // User ignores changes
            }
        });
    }
};

/**
 * Set the title of the modal dialog
 * @param {String} title The title to display into the dialog title bar
 * @returns {Boolean} Value when succeeded, false otherwise
 */
znetdkMobile.modal.setTitle = function (title) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Modal is not instantiated!');
        return false;
    }
    var titleElement = this.element.find('header .title');
    if (titleElement.length === 1) {
        titleElement.html(title);
        return true;
    }
    return false;
};

/**
 * Handle Click events on the modal close buttons and then close the modal
 */
znetdkMobile.modal.events.handleAllClose = function () {
    var modalClass = '.' + znetdkMobile.modal.cssClass,
            selector = modalClass + ' span.close, '
            + modalClass + ' button.cancel';
    $('body').on('click.znetdkmobile_modal', selector, function () {
        var modalObject = znetdkMobile.modal.make($(this).closest(modalClass));
        if (modalObject !== null
                && modalObject.events.triggerBeforeClose.call(modalObject) !== false) {
            modalObject.close(true);
        }
    });
};

/**
 * Trigger the before close event when a modal dialog is closed
 * @returns {Boolean}
 */
znetdkMobile.modal.events.triggerBeforeClose = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Modal is not instantiated!');
        return null;
    }
    var status = this.element.triggerHandler(this.events.beforeCloseNane);
    return status !== false;
};

//*************************** FORM PUBLIC METHODS ******************************

/**
 * Instantiate a new form object from the specified HTML form element.
 * @param {String|jQuery} formElementSelector The form element as a jQuery
 * element or a string for selecting a HTML form
 * @param {function} submitCallback A function to call back when the submit
 * button is pressed
 * @returns {Object} The form object
 */
znetdkMobile.form.make = function (formElementSelector, submitCallback) {
    var formElement = formElementSelector instanceof jQuery
            ? formElementSelector : $(formElementSelector);
    if (formElement.length !== 1) {
        znetdkMobile.log.error('Form element is empty or multiple!');
        return null;
    }
    if (!formElement.is('form')) {
        znetdkMobile.log.error('Not a form element!', formElement);
        return null;
    }
    var formObject = Object.create(this);
    formObject.element = formElement;
    znetdkMobile.ajax.initControllerActionProperties('submit', formObject);
    znetdkMobile.ajax.initControllerActionProperties('load', formObject);
    formObject.inputData = new FormData(formElement[0]);
    _registerSubmitCallback(submitCallback);
    return formObject;

    function _registerSubmitCallback(submitCallback) {
        if (typeof submitCallback !== 'function') {
            return false;
        }
        var eventName = formObject.events.afterSubmitSuccessName + '.znetdkmobile_form';
        formElement.off(eventName);
        formElement.on(eventName, function (event, response) {
            return submitCallback(response);
        });
    }
};

/**
 * Set the focus on the first input field into the form
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.setFocusOnFirstInput = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    var selector = 'textarea:visible:not([disabled]):not([readonly])'
        + ',select:visible:not([disabled]):not([readonly])'
        + ',input:visible:not([disabled]):not([readonly])',
        focusedElement = this.element.find(selector).first();
    if (focusedElement.length === 1) {
        focusedElement.trigger('focus');
        return true;
    }
    return false;
};

/**
 * Set the focus on the specified input field
 * @param {String} inputName Name of input field ('name' HTML attribute)
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.setFocus = function (inputName) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    if (typeof inputName === 'string') {
        var focusedElement = this.element.find('input[name=' + inputName + ']'
                + ',textarea[name=' + inputName + ']'
                + ',select[name=' + inputName + ']');
        if (focusedElement.length === 1) {
            focusedElement.focus().select();
            return true;
        }
    }
    return false;
};

/**
 * Set the modified data state of a form
 * @param {Boolean} isModified If true, the form data are set modified.
 *  Otherwise the form data are set not modified.
 * @returns {Boolean} Value true on success. False if the parameter is not a
 * boolean value.
 */
znetdkMobile.form.setDataModifiedState = function (isModified) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    if (typeof isModified !== 'boolean') {
        return false;
    }
    this.element.data('is-modified', isModified ? 'y' : 'n');
    return true;
};

/**
 * Indicates whether the form data are modified by the user
 * @returns {Boolean} Value true if the form data are modified by the user
 */
znetdkMobile.form.isModified = function() {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    return this.element.data('is-modified') === 'y';
};

/**
 * Reset the form by clearing the input fields and hidding the displayed error
 * message.
 * The hidden input fields are also reset.
 * @returns {Boolean} Value true when the input form is reset, false otherwise.
 */
znetdkMobile.form.reset = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    this.hideError();
    this.element[0].reset();
    // Hidden fields are also reset
    this.element.find('input[type=hidden]').val('');
    // The form data state is set to not modified
    this.setDataModifiedState(false);
    return true;
};

/**
 * Initialize the data form from the specified value object
 * @param {Object} valueObject A Javascript object having properties named
 * as each input field name of the form
 * @param {Boolean} isFormResetBefore When set to true or undefined, the form
 * values are cleared before initialization.
 * @returns {Boolean} Value true if initialization succeeded.
 */
znetdkMobile.form.init = function (valueObject, isFormResetBefore) {
    if (valueObject === null || typeof valueObject !== 'object') {
        znetdkMobile.log.error('Not a value object!');
        return false;
    }
    if (Object.keys(valueObject).length === 0) {
        znetdkMobile.log.error('The value object is empty!');
        return false;
    }
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    if (typeof isFormResetBefore === 'undefined' || isFormResetBefore === true) {
        this.reset();
    }
    for (var inputName in valueObject) {
        this.setInputValue(inputName, valueObject[inputName], true);
    }
    return true;
};

/**
 * Load the form data from the specified ID and the remote controller
 * action specified through the 'data-zdk-load' HTML attribute
 * @param {string} id Identifier of the data to load into the form
 * @param {function} callback The callback function to execute once the data
 * are loaded in the form
 * @returns {Boolean} Value true if the 'data-zdk-load' is defined, false
 * otherwise
 */
znetdkMobile.form.load = function (id, callback) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    if (!this.hasOwnProperty('remoteActions')
            || !this.remoteActions.hasOwnProperty('load')
            || !this.remoteActions.load.hasOwnProperty('controller')
            || !this.remoteActions.load.hasOwnProperty('action')
            || typeof this.remoteActions.load.controller !== 'string'
            || typeof this.remoteActions.load.action !== 'string') {
        znetdkMobile.log.error('The data loading controller action is not properly set!');
        return false;
    }
    var $this = this;
    znetdkMobile.ajax.request({
        control: this.remoteActions.load.controller,
        action: this.remoteActions.load.action,
        data: {id: id},
        callback: function (response) {
            if ($this.init(response) && typeof callback === 'function') {
                callback(response);
            }
        }
    });
    return true;
};

/**
 * Display an error message on the top of the form
 * @param {String} message The error message
 * @param {String} inputName The name of the input on which the focus is set (in
 * option)
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.showError = function (message, inputName) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    this.element.prepend('<div class="alert w3-panel w3-red"><p><i class="fa fa-warning"></i>&nbsp;&nbsp;' + message + '</p></div>');
    if (typeof inputName !== 'undefined') {
        this.setFocus(inputName);
    }
    this.element[0].scrollIntoView();
    return true;
};

/**
 * Hide the error message currently displayed into the form
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.hideError = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    this.element.find('div.alert').remove();
    return true;
};

/**
 * Indicates whether the specified input field exists or not
 * @param {String} inputName Name given to the input through the HTML name
 * attribute
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.doesInputExist = function (inputName) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    return typeof inputName === 'string'
            && this.element.find('[name=' + inputName + ']').length > 0;
};

/**
 * Get the value of the specified input field
 * @param {String} inputName Name of the input
 * @returns {String|Boolean} The value of the input field, false otherwise
 */
znetdkMobile.form.getInputValue = function (inputName) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    if (typeof inputName !== 'string') {
        znetdkMobile.log.error('String type expected for the inputName parameter!');
        return false;
    }
    var inputElement = this.element.find('[name="' + inputName + '"]');
    if (inputElement.length === 0) {
        znetdkMobile.log.error("No input found with name='" + inputName + "'!");
        return false;
    }
    if (inputElement.is('input')) {
        if (inputElement.length === 1) {
            var inputTypes = ['text', 'color', 'date', 'datetime-local', 'email',
                'hidden', 'month', 'number', 'password', 'range', 'search', 'tel',
                'time', 'url', 'week'],
                    currentInputType = inputElement.attr('type');
            if (inputTypes.indexOf(currentInputType) !== -1) {
                return inputElement.val();
            } else if (currentInputType === 'checkbox') {
                return inputElement.is(':checked') ? inputElement.val() : '';
            }
            znetdkMobile.log.error("The input type='" + currentInputType + "' is not supported!");
            return false;
        } else { // Multiple input elements with the same name (radio buttons)
            var radioValueFound = null,
                radioButtonCount = 0;
            inputElement.each(function () {
                if ($(this).attr('type') === 'radio' && $(this).prop('checked') === true) {
                    radioValueFound = $(this).val();
                    return false;
                }
                if ($(this).attr('type') === 'radio') {
                    radioButtonCount++;
                }
            });
            if (radioValueFound !== null) {
                return radioValueFound;
            } else if (inputElement.length === radioButtonCount) {
                // No radio button is selected
                return radioValueFound;
            }
            znetdkMobile.log.error("The multiple input element with name='" + inputName + "' is not supported!");
            return false;
        }
    }
    if (inputElement.is('select')) {
        if (inputElement.prop('multiple')) {
            let selectedItems = [];
            inputElement.find('option:selected').each(function () {
                selectedItems.push($(this).val());
            });
            return selectedItems;
        }
        return inputElement.val();
    }
    if (inputElement.is('textarea')) {
        return inputElement.val();
    }
    znetdkMobile.log.error("The input with name='" + inputName + "' is not supported!");
    return false;
};

/**
 * Set the value of the specified input field
 * @param {String} inputName Name of the input field for which setting the value
 * @param {String|Array} inputValue Value(s) to set to the input field
 * @param {Boolean} silent When set to false, an error is displayed into the
 * browser console if the specified input field not exists
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.setInputValue = function (inputName, inputValue, silent) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    if (typeof inputName !== 'string') {
        znetdkMobile.log.error('The input name must be a string!');
        return false;
    }
    var inputElement = this.element.find('[name="' + inputName + '"]');
    if (inputElement.length === 0) {
        if (silent !== true) {
            znetdkMobile.log.error("No element found in the DOM with the name '" + inputName + "'!");
        }
        return false;
    }
    if (inputElement.is('input')
            && (inputElement.attr('type') === 'radio'
                    || inputElement.attr('type') === 'checkbox')) {
        // RADIO BUTTON OR CHECKBOK
        if (inputValue === '') {
            return false; // Empty value
        }
        var checkedElement = inputElement.filter('[value="' + inputValue + '"]');
        if (checkedElement.length === 1) {
            checkedElement.prop('checked', true);
            return true;
        }
        if (inputElement.attr('type') === 'radio') {
            znetdkMobile.log.error("The input element of type '" + inputElement.attr('type')
                    + "' and named '" + inputName
                    + "' does not exist with the value='" + inputValue + "'!");
            return false;
        }
        // CHECKBOX
        inputElement.prop('checked', false); // Forced to unchecked
        if (Array.isArray(inputValue) && inputValue.length > 0) {
            $.each(inputValue, function () {
                var selectedInput = inputElement.filter('[value="' + this + '"]');
                if (selectedInput.length === 1) {
                    selectedInput.prop('checked', true);
                }
            });
        }
        return true;
    } else if (inputElement.is('select') && inputElement.prop('multiple') === true) {
        // SELECT MULTIPLE
        if (!Array.isArray(inputValue)) {
            if (silent !== true) {
                znetdkMobile.log.error("The value for the multiple select element with name='"
                        + inputName + "' must be an array!", inputValue);
            }
            return false;
        }
        $.each(inputValue, function () {
            var selectedOption = inputElement.find('option[value="' + this + '"]');
            if (selectedOption.length === 1) {
                selectedOption.prop('selected', true);
            }
        });
        inputElement.scrollTop(0);
        return true;
    } else {
        // OTHERS
        inputElement.val(inputValue);
        return true;
    }
};

/**
 * Set the form readonly
 * @param {Boolean} isReadOnly When set to true, the form values can't be
 *  changed. If set to false, the form values so can be changed.
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.form.setReadOnly = function (isReadOnly) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return false;
    }
    var readOnlyState = typeof isReadOnly === 'undefined' || isReadOnly === true;
    this.element.find('input:not([type=hidden]), textarea').each(function () {
        $(this).prop('readonly', readOnlyState);
    });
    this.element.find('select').each(function () {
        // a select element can't be set readonly, so it is disabled
        $(this).prop('disabled', readOnlyState);
    });
    this.element.find('button[type=submit], input[type=submit]').prop('disabled', readOnlyState);
};

/**
 * Handle the input events of the forms and update their modified data state
 * (see HTML 'data-is-modified' attribute).
 */
znetdkMobile.form.events.handleAllInput = function () {
    $('body').on('input.znetdkmobile_form_submit', 'form[data-zdk-submit]', function (event) {
        if ($(event.target).is(':input')) {
            var formElement = $(this),
                    formObject = znetdkMobile.form.make(formElement);
            formObject.setDataModifiedState(true);
        }
    });
};

/**
 * Handle the submit events of the forms by sending the form data in AJAX when
 * a controller and an action are set through the 'data-zdk-submit' HTML5
 *  attribute.
 */
znetdkMobile.form.events.handleAllSubmit = function () {
    $('body').on('submit.znetdkmobile_form_submit', 'form[data-zdk-submit]', function (event) {
        var formElement = $(this),
                formObject = znetdkMobile.form.make(formElement);
        znetdkMobile.ajax.request({
            control: formObject.remoteActions.submit.controller,
            action: formObject.remoteActions.submit.action,
            data: formObject.inputData,
            callback: function (response) {
                formObject.hideError();
                if (formObject.events.triggerAfterSubmitSuccess.call(formObject, response) === false) {
                    return false;
                }
                if (response.success === false) {
                    formObject.showError(response.msg, response.ename);
                } else if (response.hasOwnProperty('msg') && response.hasOwnProperty('summary')) {
                    var severity = response.hasOwnProperty('success') && response.success ? 'info' : 'error';
                    if (response.hasOwnProperty('warning') && response.warning) {
                        severity = 'warn';
                    }
                    if (response.summary === null) {
                        znetdkMobile.messages.showSnackbar(response.msg, response.warning);
                    } else {
                        znetdkMobile.messages.add(severity, response.summary, response.msg);
                    }
                }
            }
        });
        event.preventDefault();
    });
};

/**
 * Trigger the aftersubmitsuccess event when the form data is sent successfully
 * to the remote controller action.
 * @param {type} response The response of the remote controller action
 * @returns {Boolean|null} Value null if form is not instantiated, true or false
 * according to the value returned by the event handler defined by the developer
 */
znetdkMobile.form.events.triggerAfterSubmitSuccess = function (response) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('Form is not instantiated!');
        return null;
    }
    var status = this.element.triggerHandler(this.events.afterSubmitSuccessName, [response]);
    return status !== false;
};

/**
 * Handle the click events of the show/hide button on the password input fields
 */
znetdkMobile.form.events.handleTogglePassword = function () {
    $('body').on('click.znetdkmobile_form_password', 'a.zdk-toggle-password', function (event) {
        var passwordInput = $(this).prev('input'),
                inputType = passwordInput.attr('type');
        if (inputType === 'password') {
            passwordInput.attr('type', 'text');
            $(this).children('i.fa').removeClass('fa-eye-slash').addClass('fa-eye');
        } else if (inputType === 'text') {
            passwordInput.attr('type', 'password');
            $(this).children('i.fa').removeClass('fa-eye').addClass('fa-eye-slash');
        }
        event.preventDefault();
    });
};

//************************** ACTION PUBLIC METHODS *****************************

/**
 * Specify the action buttons to display each time the specified view is
 * displayed and the function to call back when the action button is pressed
 * @param {String} viewId ID of the view
 * @param {Object} options For each button set as property 'add', 'refresh',
 *  'search' and 'scrollup', the sub-properties 'isVisible' (boolean) and
 *  'callback' (function)
 * @returns {Boolean} Value true if the options are set properly and the view
 * exists, false otherwise
 */
znetdkMobile.action.registerView = function (viewId, options) {
    if (options === null || typeof options !== 'object') {
        znetdkMobile.log.error("The 'options' parameter is not an object!");
    }
    var $this = this,
            isOptionOk = false;
    $.each(options, function (button, properties) {
        if (!$this.buttons.hasOwnProperty(button)
                || !properties.hasOwnProperty('isVisible')
                || typeof properties.isVisible !== 'boolean') {
            isOptionOk = false;
            return false; // break
        }
        isOptionOk = true;
    });
    if (isOptionOk === false) {
        znetdkMobile.log.error("The 'option' properties are invalid!");
        return false;
    }
    if (znetdkMobile.content.doesViewExistInDom(viewId)) {
        if (!$this.views.hasOwnProperty(viewId)) {
            $this.views[viewId] = {};
        }
        $.each(options, function (button, properties) {
            $this.views[viewId][button] = properties;
        });
        return true;
    }
    znetdkMobile.log.error("The view '" + viewId + "' does not exist in the DOM!");
    return false;
};

/**
 * Display the scroll to top action button for the specified view
 * @param {String} viewId ID of the view
 * @returns {Boolean} Value true when the view exists, false otherwise
 */
znetdkMobile.action.setScrollUpButtonForView = function (viewId) {
    return this.registerView(viewId, {
        scrollup: {
            isVisible: true,
            callback: function () {
                $("html, body").animate({scrollTop: 0}, 'fast');
            }
        }
    });
};

/**
 * Show or hide the action buttons for the view currently displayed
 * @returns {Boolean} Value true when action buttons are shown or hidden for
 * the view currently displayed, false otherwise
 */
znetdkMobile.action.toggle = function () {
    var $this = this, viewId = znetdkMobile.content.getDisplayedViewId();
    if (viewId === null) {
        return false;
    }
    if (!this.views.hasOwnProperty(viewId)) {
        this.hide(); // View not registered, action buttons are all hidden
        return false;
    }
    var footerHeight = znetdkMobile.footer.getHeight(),
            verticalPos = footerHeight - 10;
    $.each(this.buttons, function (button, properties) {
        if ($this.views[viewId].hasOwnProperty(button)
                && $this.views[viewId][button].isVisible) { // Button is set visible
            // Position is adjusted
            $(properties.id).css({
                bottom: verticalPos,
                right: 5
            });
            verticalPos += $this.buttonGap;
            // Button is shown
            $($this.buttons[button].id).removeClass('w3-hide');
        } else { // Button is set hidden
            $($this.buttons[button].id).addClass('w3-hide');
        }
    });
    return true;
};
/**
 * Hide all the action buttons
 */
znetdkMobile.action.hide = function () {
    $.each(this.buttons, function () {
        var buttonId = this.id;
        $(buttonId).addClass('w3-hide');
    });
};

/**
 * Adjust the position of the action buttons
 * @return {Boolean} Value true when action buttons position is adjusted for
 * the view currently displayed, false otherwise
 */
znetdkMobile.action.adjustPosition = function () {
    return this.toggle();
};

/**
 * Add a custom action button
 * @param {String} name Name given to the custom button.
 * @param {String} iconCssClass Class name of the icon displayed on the button
 * @param {String} colorCssClass Class name of the color used to display the
 * button
 * @returns {Boolean} Value true on success, false otherwise
 */
znetdkMobile.action.addCustomButton = function (name, iconCssClass, colorCssClass) {
    if (this.buttons.hasOwnProperty(name)) {
        znetdkMobile.log.error("The '" + name + "' button name already exists!");
        return false;
    }
    var buttonIdPrefix = 'zdk-mobile-action-custom-',
        buttonIdSuffix = 1,
        buttonId = '';
    while ($('#'+buttonIdPrefix+buttonIdSuffix).length > 0) {
        buttonIdSuffix++;
    }
    buttonId = buttonIdPrefix + buttonIdSuffix;
    $('#zdk-mobile-action-search').after(
            '<a id="' + buttonId
            + '" class="zdk-mobile-action ' + name + ' w3-hide w3-btn w3-circle w3-ripple w3-xlarge '
            + colorCssClass + ' w3-card-4"><i class="fa '
            + iconCssClass + '"></i></a>');
    this.buttons[name] = {id: '#'+buttonId};
    return true;
};

/**
 * Handle the click events of the action buttons and execute the callback
 *  function when an action button is clicked
 */
znetdkMobile.action.events.handleClick = function () {
    $('body').on('click.znetdkmobile_action', 'a.zdk-mobile-action', function (event) {
        event.preventDefault();
        var viewId = znetdkMobile.content.getDisplayedViewId();
        if (viewId === null) {
            znetdkMobile.log.error("Unable to identify the ID of the currently displayed view.");
            return false;
        }
        if (!znetdkMobile.action.views.hasOwnProperty(viewId)) {
            //View not registered
            znetdkMobile.log.error("The view ID='" + viewId + "' is not registered.");
            return false;
        }
        var buttonId = '#' + $(this).attr('id'), buttonKey = null;
        $.each(znetdkMobile.action.buttons, function (key, properties) {
            if (properties.id === buttonId) {
                buttonKey = key;
                return false; // Break the loop
            }
        });
        if (buttonKey === null) {
            znetdkMobile.log.error("No action button definition found for the ID='" + buttonId + "'!");
            return false;
        }
        if (!znetdkMobile.action.views[viewId][buttonKey].hasOwnProperty('callback')) {
            znetdkMobile.log.error("No callback function set for the '" + buttonKey + "' button of the view ID='" + viewId + "'.");
            return false;
        }
        // The callback method is executed
        znetdkMobile.action.views[viewId][buttonKey].callback();
    });
};

//*************************** LIST PUBLIC METHODS ******************************

/**
 * Instantiate the list object for the specified UL HTML element
 * @param {String} listElementId Identifier ('id' HTML attribute value) of the
 *  UL HTML element
 * @param {Boolean} refresh If true or undefined, the 'refresh' action button
 *  is displayed for refreshing the list content on demand
 * @param {Boolean} search If true or undefined, the 'search' action button is
 * displayed for filtering/sorting the list content
 * @returns {Object|null} The instantiated object for the list
 */
znetdkMobile.list.make = function (listElementId, refresh, search) {
    if (typeof listElementId !== 'string') {
        znetdkMobile.log.error('List element ID is not a string!');
        return null;
    }
    if ($(listElementId).length !== 1) {
        znetdkMobile.log.error('List element is empty!');
        return null;
    }
    if (!$(listElementId).is('ul')) {
        znetdkMobile.log.error('Not a List!', $(listElementId));
        return null;
    }
    var listObject = Object.create(this);
    listObject.element = $(listElementId);
    var firtRows = listObject.element.children();
    if (firtRows.length === 0) {
        znetdkMobile.log.error('No row found within the List!', $(listElementId));
        return null;
    } else if (firtRows.length > 2) {
        znetdkMobile.log.error('More than 2 rows are defined!', $(listElementId));
    }
    if (!firtRows.first().is('li')) {
        znetdkMobile.log.error('The first row element is not a list item!', firtRows.first());
    }
    if (firtRows.length === 2 && !firtRows.eq(1).is('li')) {
        znetdkMobile.log.error('The second row element is not a list item!', firtRows.first());
    }
    if (znetdkMobile.ajax.initControllerActionProperties('load', listObject) === false) {
        znetdkMobile.log.error("The 'data-zdk-load' attribute is missing!");
        return null;
    }
    // Memorize the custom 'No row found' message and remove it
    if (firtRows.length === 2) {
        listObject.noRowMessage = '<li>' + firtRows.eq(1).html() + '</li>';
        firtRows.eq(1).remove();
    }
    // Memorize the row template
    listObject.rowTemplate = listObject.element.html();
    // View is registered for action buttons
    znetdkMobile.action.registerView(
            znetdkMobile.content.getParentViewId(listObject.element),
            {
                search: {
                    isVisible: search === true || typeof search === 'undefined',
                    callback: function () {
                        listObject.showSearchModal();
                    }
                },
                refresh: {
                    isVisible: refresh === true || typeof refresh === 'undefined',
                    callback: function () {
                        listObject.refresh();
                    }
                }
            }
    );
    // Add the filter buttons memorized into the local storage
    addTagFromLocalStorage();
    // 'Display view' events are handled for refreshing the list
    var afterViewDisplayEventName = znetdkMobile.navigation.events.afterViewDisplayName
            + '.znetdkmobile_list_' + znetdkMobile.content.getParentViewId(listObject.element);
    $('body').off(afterViewDisplayEventName);
    $('body').on(afterViewDisplayEventName, function (event, viewId) {
        var thisViewId = znetdkMobile.content.getParentViewId(listObject.element);
        if (thisViewId === viewId) {
            listObject.refresh();
        }
    });
    // 'Click' Events of the Remove filter buttons are handled for refreshing the list
    if (search === true || typeof search === 'undefined') {
        listObject.element.parent().on('click.znetdkmobile_list', '.search-filter-container .remove', function () {
            var searchTag = $(this).closest('.search-tag'),
                filterContainer = $(this).closest('.search-filter-container');
            // The tag is cleared into the local storage
            clearTagIntoLocalStorage(searchTag);
            // The tag is removed
            searchTag.remove();
            // The filter container is removed if it is empty
            if (filterContainer.is(':empty')) {
                filterContainer.remove();
            }
            // The list is refreshed
            listObject.refresh();
        });
    }
    // Autocomplete suggestions on search field if data-zdk-autocomplete is set for the list
    listObject.isAutocompleteOnSearchEnabled = znetdkMobile.ajax.initControllerActionProperties(
            'autocomplete', listObject);

    return listObject;
    /**
     * Adds the search filter tags from the local storage
     */
    function addTagFromLocalStorage() {
        var listId = listObject.element.attr('id'),
            viewId = znetdkMobile.content.getParentViewId(listObject.element),
            storageKeySuffix = '_' + listId + '_' + viewId,
            sortField = znetdkMobile.browser.readLocalData(listObject.sortFieldStorageKey + storageKeySuffix),
            sortOrder = znetdkMobile.browser.readLocalData(listObject.sortOrderStorageKey + storageKeySuffix),
            sortLabel = znetdkMobile.browser.readLocalData(listObject.sortLabelStorageKey + storageKeySuffix),
            keywords = znetdkMobile.browser.readLocalData(listObject.keywordsStorageKey + storageKeySuffix);

        if (keywords !== false) {
            // Memorized keywords are applied
            var keywordsAsArray = keywords.split('&&');
            keywordsAsArray.forEach(function(keyword) {
                listObject.applyFilterAndSortCriterium(keyword, null, null, null, true);
            });
        }
        if (sortField !== false && sortOrder !== false && sortLabel !== false) {
            // Memorized Sort criteria are applied
            listObject.applyFilterAndSortCriterium('', sortField, sortOrder, sortLabel, true);
        }
    }
    /**
     * Removes from the local storage the data of the removed search tag
     * @param {jQuery} searchTag search tag element
     */
    function clearTagIntoLocalStorage(searchTag) {
        var listId = listObject.element.attr('id'),
            viewId = znetdkMobile.content.getParentViewId(listObject.element),
            storageKeySuffix = '_' + listId + '_' + viewId;
        if (searchTag.is('.filter-sort-criterium')) {
            // The sort criterium are cleared
            znetdkMobile.browser.removeLocalData(listObject.sortFieldStorageKey + storageKeySuffix);
            znetdkMobile.browser.removeLocalData(listObject.sortOrderStorageKey + storageKeySuffix);
            znetdkMobile.browser.removeLocalData(listObject.sortLabelStorageKey + storageKeySuffix);
        } else if (searchTag.is('.search-filter')) {
            var doRemoveFilterTag = true;
            // The keyword is cleared
            if (listObject.uniqueSearchedKeyword !== true) {
                var storedKeywords = znetdkMobile.browser.readLocalData(listObject.keywordsStorageKey + storageKeySuffix);
                if (storedKeywords === false) {
                    return; // No stored keyword
                }
                var keywordsAsArray = storedKeywords.split('&&'),
                        keywordToClear = searchTag.find('.filter-value').text(),
                        keywordWithJsonDataToClear = '[[' + searchTag.attr('data-filter-jsondata') + ']]'
                            + keywordToClear,
                        newStoredKeywords = '';
                keywordsAsArray.forEach(function(keyword) {
                    if (keyword !== keywordToClear && keyword !== keywordWithJsonDataToClear) {
                        newStoredKeywords += newStoredKeywords === '' ? keyword : '&&' +  keyword;
                    }
                });
                if (newStoredKeywords !== '') {
                    znetdkMobile.browser.storeLocalData(listObject.keywordsStorageKey + storageKeySuffix, newStoredKeywords);
                    doRemoveFilterTag = false;
                }
            }
            if (doRemoveFilterTag) {
                // Only one keyword is applied, so all keywords are removed
                znetdkMobile.browser.removeLocalData(listObject.keywordsStorageKey + storageKeySuffix);
            }
        }
    }
};

/**
 * Memorize the next page number of data to load in the list when the list
 * is scrolled down.
 * This method is used to simulate the infinite scroll mechanism
 * @param {Integer} pageNumber The page number
 * @returns {Boolean} Value true when the method is called in the context of a
 * list object, false otherwise.
 */
znetdkMobile.list.setNextPageToLoad = function (pageNumber) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    this.element.data('page', pageNumber);
    return true;
};

/**
 * Get the next page number of data to load in the list when the list is
 * scrolled down.
 * This method is used to simulate the infinite scroll mechanism
 * @returns {Integer|false} The page number when succeeded, false when called
 * out of the context of a list object
 */
znetdkMobile.list.getNextPageToLoad = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    return parseInt(this.element.data('page'));
};

/**
 * Memorize the HTML document height in pixels after the loading of extra rows
 *  in a list.
 * This method is used to simulate the infinite scroll mechanism
 * @param {Float} documentHeight The height of the document in pixels
 * @returns {Boolean} Value true when called in the context of a list object,
 * false otherwise.
 */
znetdkMobile.list.setLastDocumentHeight = function (documentHeight) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    this.element.data('height', documentHeight);
    return true;
};

/**
 * Get the last memorized document height of the application
 * This method is used to simulate the infinite scroll mechanism
 * @returns {Float|false} The number of pixels of false if the method is not
 * called in the context of a list object
 */
znetdkMobile.list.getLastDocumentHeight = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    return parseFloat(this.element.data('height'));
};

/**
 * Memorize the total number of rows loaded into the list
 * This method is used to simulate the infinite scroll mechanism
 * @param {Integer} rowCount Number of rows
 * @returns {Boolean} Value true when called in the context of a list object,
 * false otherwise.
 */
znetdkMobile.list.setTotalRowCount = function (rowCount) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    this.element.data('total', rowCount);
    return true;
};

/**
 * Get the number of rows loaded into the list
 * This method is used to simulate the infinite scroll mechanism
 * @returns {Integer|false} The number of rows of false if the method is not
 * called in the context of a list object
 */
znetdkMobile.list.getTotalRowCount = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    return parseInt(this.element.data('total'));
};

/**
 * Filter and sort the list's rows
 * The filter keywords and sort criteria are memorized into the local storage
 * @param {String} filterValue The keyword for filtering the list content
 * @param {String} sortfield The field name to sort by. When is equal to
 * '_default', no sort is applied.
 * @param {String} sortorder The sort order of the data ('1' for ascending order
 *  or '-1' for descending order)
 *  @param {String} sortlabel The label of the sort field name (optional, set
 *  only when the sort filter is applied from the local storage)
 *  @param {Boolean} noRefresh When set to true, the list is not refreshed
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.list.applyFilterAndSortCriterium = function (filterValue,
        sortfield, sortorder, sortlabel, noRefresh) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    var isSortCriteriumToApply = sortfield !== null && sortfield !== '_default'
            && (sortfield !== this.defaultCustomSortCriterium || sortorder !== '1'),
            filterContainer = this.element.parent().children('.search-filter-container');

    // Displayed sort criterium are removed
    filterContainer.find('.filter-sort-criterium').remove();

    // Displayed keyword are removed
    if (this.uniqueSearchedKeyword === true) {
        filterContainer.find('.search-filter').remove();
    }
    if (filterValue !== '' || isSortCriteriumToApply) {
        if (filterContainer.length === 0) {
            filterContainer = $(this.searchModalId).find('.search-filter-container').clone();
            filterContainer.removeClass('w3-hide');
            filterContainer.insertBefore(this.element);
        }
        // Display keyword filters
        if (filterValue !== '') {
            var filterElement = $(this.searchModalId).find('.search-filter').clone();
            filterElement.find('.filter-value').text(getFilterValueLabel());
            filterElement.attr('data-filter-jsondata', getFilterValueJsonData());
            filterElement.attr('data-filter-key', 'keyword');
            filterElement.appendTo(filterContainer);
            filterElement.removeClass('w3-hide');
        }
        // Display sort criterium
        if (isSortCriteriumToApply) {
            var orderClass = sortorder === '1' ? 'asc' : 'desc',
                    sortElement = $(this.searchModalId).find('.filter-sort-criterium.' + orderClass).clone();
            sortElement.find('.label').text(
                    typeof sortlabel === 'undefined'
                        ? this.customSortCriteria[sortfield] : sortlabel);
            sortElement.attr('data-sortfield', sortfield);
            sortElement.attr('data-sortorder', sortorder);
            sortElement.appendTo(filterContainer);
            sortElement.removeClass('w3-hide');
        }
        if (noRefresh === true) {
            return true;
        } else {
            // The filters are memorized
            addFiltersToLocalStorage(this);
            // The list content is refresh
            return this.refresh();
        }
    }
    return false;
    function getFilterValueLabel() {
        var endDataSeparatorPos = filterValue.indexOf(']]');
        if (endDataSeparatorPos === -1) {
            return filterValue;
        }
        return filterValue.substring(endDataSeparatorPos + 2);
    }
    function getFilterValueJsonData() {
        var startDataSeparatorPos = filterValue.indexOf('[['),
                endDataSeparatorPos = filterValue.indexOf(']]');
        if (startDataSeparatorPos === -1 || endDataSeparatorPos === -1) {
            return {label:filterValue};
        }
        return filterValue.substring(startDataSeparatorPos + 2, endDataSeparatorPos);
    }
    /**
     * Stores in the local storage the filter and sort criteria
     * @param listContext The list context
     */
    function addFiltersToLocalStorage(listContext) {
        var listId = listContext.element.attr('id'),
                viewId = znetdkMobile.content.getParentViewId(listContext.element),
                storageKeySuffix = '_' + listId + '_' + viewId;
        if (isSortCriteriumToApply) {
            // Sort criteria are stored
            var sortTag = listContext.element.parent().find('.filter-sort-criterium'),
                    sortLabel = sortTag.find('span.label').text();

            znetdkMobile.browser.storeLocalData(listContext.sortFieldStorageKey + storageKeySuffix, sortfield);
            znetdkMobile.browser.storeLocalData(listContext.sortOrderStorageKey + storageKeySuffix, sortorder);
            znetdkMobile.browser.storeLocalData(listContext.sortLabelStorageKey + storageKeySuffix, sortLabel);
        }
        if (filterValue !== '') {
            var keywords = filterValue;
            if (listContext.uniqueSearchedKeyword === false) {
                var storedKeywords = znetdkMobile.browser.readLocalData(listContext.keywordsStorageKey + storageKeySuffix);
                if (storedKeywords !== false) {
                    keywords = storedKeywords + '&&' + filterValue;
                }
            }
            znetdkMobile.browser.storeLocalData(listContext.keywordsStorageKey + storageKeySuffix, keywords);
        }
    }
};

/**
 * Refresh the list from the data returned by the remote controller action set
 * for the list through the 'data-zdk-load' HTML5 attribute
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.list.refresh = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    // The footer is temporarely hidden
    znetdkMobile.footer.hide();
    // Rows are removed first
    this.element.find('li').remove();
    // The list is shown if initialy hidden
    this.element.removeClass('w3-hide');
    // Pagination infos are reset
    this.setNextPageToLoad(1);
    this.setTotalRowCount(this.rowsPerPage);
    this.setLastDocumentHeight(0);
    // The rows are downloaded from the controller action
    this.loadNewDataPage();
    return true;
};

/**
 * Load a new page of data rows into the list
 * This method is used to simulate the infinite scroll mechanism
 * @returns {Boolean} Value true when succeeded, false otherwise
 */
znetdkMobile.list.loadNewDataPage = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    if (this.remoteActions.load.controller === null || this.remoteActions.load.action === null) {
        znetdkMobile.log.error('Controller or action property not set!');
        return false;
    }
    var $this = this,
            nextPage = this.getNextPageToLoad(),
            first = (nextPage - 1) * this.rowsPerPage,
            requestData = {first: first, count: this.rowsPerPage};
    if (first >= this.getTotalRowCount()) {
        znetdkMobile.footer.adjustPosition(); // Footer was hidden so it is displayed
        return false; // No more rows to load
    }
    // Scroll events are handled for loading data pages (infinite scroll)
    var eventName = this.events.onContentScrollName + '.znetdkmobile_list';
    if (nextPage === 1) { // First page to load so no scroll event handled
        this.element.off(eventName);
    }
    // If filters are set, they are added to the request Data
    addFiltersToRequestData.call(this, requestData);
    znetdkMobile.ajax.request({
        control: this.remoteActions.load.controller,
        action: this.remoteActions.load.action,
        data: requestData,
        callback: function (response) {
            if (response.rows.length > 0) {
                $this.setTotalRowCount(response.total);
                $this.setNextPageToLoad(nextPage + 1);
                $.each(response.rows, function () {
                    appendRow($(this)[0]);
                });
                if (nextPage === 1 && response.total > $this.rowsPerPage) {
                    // Data pagination enabled, data pages are loaded on scroll
                    handleScrollEvents();
                }
            } else if (nextPage === 1) {
                // No result found
                $this.element.append($this.noRowMessage);
            }
            // The total number of rows is displayed on the horizontal menu item
            znetdkMobile.navigation.addRowCountToHorizontalMenuItem(
                    response.total,
                    znetdkMobile.content.getParentViewId($this.element));
            // The footer position is adjusted
            znetdkMobile.footer.adjustPosition();
            // Private functions
            function appendRow(rowData) {
                var newRowAsHtml = $this.rowTemplate;
                $.each(rowData, function (key, value) {
                    var toReplace = '{{' + key + '}}';
                    newRowAsHtml = newRowAsHtml.replace(new RegExp(toReplace, 'g'), value);
                });
                $this.element.append(newRowAsHtml);
            }
            function handleScrollEvents() {
                $this.element.on(eventName, function () {
                    var windowHeight = $(window).height(),
                            documentHeight = $(document).height(),
                            scrollBarPosition = $(window).scrollTop(),
                            newPageLevel = windowHeight + scrollBarPosition + $this.heightDiffForNewPage,
                            lastDocumentHeight = $this.getLastDocumentHeight();
                    if (newPageLevel > documentHeight && lastDocumentHeight < documentHeight) {
                        znetdkMobile.footer.hide(); // Footer temporarely hidden
                        $this.loadNewDataPage();
                        $this.setLastDocumentHeight(documentHeight);
                    }
                });
            }
        }
    });
    return true;
    // Private function
    function addFiltersToRequestData(requestData) {
        // Keywords
        var $this = this, filters = this.element.parent().find('.search-filter');
        filters.each(function () {
            var filterKey = $(this).data('filter-key'),
                    filterJsonData = $(this).data('filter-jsondata'),
                    filterValue = $(this).find('.filter-value').text();
            if (requestData.hasOwnProperty(filterKey)) {
                requestData[filterKey].push(filterValue);
            } else {
                requestData[filterKey] = [filterValue];
            }
            if ($this.searchedKeywordAsJson && typeof filterJsonData === 'object') {
                if (requestData.hasOwnProperty(filterKey + '_json')) {
                    requestData[filterKey + '_json'].push(JSON.stringify(filterJsonData));
                } else {
                    requestData[filterKey + '_json'] = [JSON.stringify(filterJsonData)];
                }
            }
        });
        // Sort Criterium
        var sortCriterium = this.element.parent().find('.filter-sort-criterium');
        if (sortCriterium.length === 1) {
            requestData['sortfield'] = sortCriterium.data('sortfield');
            requestData['sortorder'] = sortCriterium.data('sortorder');
        }
        // Request data overload
        if (typeof this.beforeSearchRequestCallback === 'function') {
            this.beforeSearchRequestCallback(requestData);
        }
    }
};

/**
 * Declare the modal dialog to display for editing an existing item or for adding
 * a new one
 * @param {String} modalElementId The identifier of the modal dialog ('id' HTML
 * attribute)
 * @param {Boolean} isFormModifiable When set to true or undefined, the row data
 * can be modified through the modal dialog and the 'add' action bar button is
 * displayed.
 * @param {function|Boolean} onAdd The function to call back for initialization
 * purpose when adding a new row, just before the modal dialog display. If set
 * to false, no 'Add' action button is added to the list.
 * @param {function} onEdit The function to call back for initialization purpose
 * when editing an existing row, just before the modal dialog display.
 * @returns {Boolean} Value true when succeeded, false if the method is not
 * called into the context of a list object
 */
znetdkMobile.list.setModal = function (modalElementId, isFormModifiable, onAdd, onEdit) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    var $this = this,
            isModifiable = isFormModifiable === true || typeof isFormModifiable === 'undefined';
    // Set Add action
    if (onAdd !== false) {
        znetdkMobile.action.registerView(znetdkMobile.content.getParentViewId(this.element),
            {
                add: {
                    isVisible: isModifiable,
                    callback: function () {
                        var modal = znetdkMobile.modal.make(modalElementId),
                                innerForm = modal.getInnerForm();
                        // Form is reset
                        innerForm.reset();
                        // Custom callback
                        if (typeof onAdd === 'function') {
                            onAdd.call(modal, innerForm);
                        }
                        // Modal is shown
                        modal.open(function () {
                            // Submit succeeded so the list is refreshed
                            $this.refresh();
                        });
                    }
                }
            }
        );
    }
    // Set Edit action
    var eventName = this.events.afterClickEditButtonName + '.znetdkmobile_list';
    this.element.off(eventName);
    this.element.on(eventName, function (event, rowId) {
        var modal = znetdkMobile.modal.make(modalElementId),
                innerForm = modal.getInnerForm();
        innerForm.load(rowId, function (response) {
            // The form inputs are set readonly if edit parameter == false
            if (!isModifiable) {
                innerForm.setReadOnly();
            }
            // Custom callback
            if (typeof onEdit === 'function') {
                onEdit.call(modal, innerForm, response);
            }
            modal.open(function () {
                // Submit succeeded so the list is refreshed
                $this.refresh();
            });
        });
    });
    return true;
};

/**
 * Display the search modal dialog for entering a keyword to search amoung the
 * list's rows and for setting the sort criterium
 * @returns {Boolean} Value true when succeeded, false if the method is not
 * called into the context of a list object
 */
znetdkMobile.list.showSearchModal = function () {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    var $this = this,
            modal = znetdkMobile.modal.make(this.searchModalId),
            innerForm = modal.getInnerForm(),
            keywordInput = $(this.searchModalId).find('input[name=keyword]'),
            sortInputs = $(this.searchModalId).find('.sort-criterium'),
            sortDropdown = sortInputs.find('select[name=sortfield]');
    // The search field is set by default 'required'
    keywordInput.prop('required', true);

    if (this.isAutocompleteOnSearchEnabled) {
        keywordInput.removeData('item');
        znetdkMobile.autocomplete.make(this.searchModalId + ' input[name=keyword]', {
            controller: this.remoteActions.autocomplete.controller,
            action: this.remoteActions.autocomplete.action
        }, onSuggestionSelected);
    } else {
        // The search field is set with HTML5 attribute 'autocomplete' on if
        // the custom autocomplete feature is disabled
        keywordInput.attr('autocomplete', 'on');
    }
    // The sort criteria dropdown is emptied
    sortDropdown.empty();
    // The sort crtieria inputs are hidden by default
    sortInputs.addClass('w3-hide');
    // If custom sort criteria are set...
    if (this.customSortCriteria !== null && typeof this.customSortCriteria === 'object') {
        // The dropdown is filled
        for (var criterium in this.customSortCriteria) {
            sortDropdown.append('<option value="' + criterium
                    + '">' + this.customSortCriteria[criterium] + '</option>');
        }
        // The sort crtieria inputs are shown
        sortInputs.removeClass('w3-hide');
        // The keyword input is no longer set required
        keywordInput.prop('required', false);
    }
    // The form is reset
    innerForm.reset();
    // Current applied sort criterium is selected
    setCurrentSortCriteriumInForm();
    // The modal is displayed
    modal.open();
    // Submit events are attached
    $(this.searchModalId + ' form').off('submit.zdkmobile_list');
    $(this.searchModalId + ' form').on('submit.zdkmobile_list', function (event) {
        var keyword = innerForm.getInputValue('keyword'),
                sortfield = innerForm.getInputValue('sortfield'),
                sortorder = innerForm.getInputValue('sortorder');
        if (keyword !== '' && $this.searchedKeywordAsJson) {
            var keywordItemData = keywordInput.data('item');
            if (typeof keywordItemData === 'object') {
                keyword = '[[' + JSON.stringify(keywordItemData) + ']]' + keyword;
            } else {
                keyword = '[[' + JSON.stringify({label:keyword}) + ']]' + keyword;
            }
        }
        $this.applyFilterAndSortCriterium(keyword, sortfield, sortorder);
        modal.close();
        event.preventDefault();
    });
    return true;
    /**
     * Set the last sort criterium applied
     * @returns {Boolean} Value true if a sort criterium was previously applied,
     * false otherwise.
     */
    function setCurrentSortCriteriumInForm() {
        var sortTag = $this.element.parent().find('.filter-sort-criterium');
        if (sortTag.length !== 1) {
            return false;
        }
        var sortfield = sortTag.data('sortfield'),
                sortorder = sortTag.data('sortorder');
        innerForm.setInputValue('sortfield', sortfield);
        innerForm.setInputValue('sortorder', sortorder);
        return true;
    };
    /**
     * When a suggestion is selected, its extra data are memorized as HTML5 data
     * property of the input element.
     * @param {Object} item The selected item property values
     */
    function onSuggestionSelected(item) {
        if ($this.searchedKeywordAsJson) {
            var storedItem = {};
            for (var property in item) {
                if (item.hasOwnProperty(property) && property !== 'label') {
                    storedItem[property] = item[property];
                }
            }
            if (Object.keys(storedItem).length > 0) {
                $(this).data('item', storedItem);
            } else {
                $(this).removeData('item');
            }
        }
    }
};

/**
 * Define the sort criteria displayed within the search modal dialog for sorting
 * the list's rows
 * @param {Object} sortCriteria An object with properties as sorting key and
 * values as label to display to the users
 * @param {String} defaultSortCriterium The sorting key to apply by default
 * @returns {Boolean} Value true when succeeded, false if the method is not
 *  called into the context of a list object or if its parameter are not
 *  properly set
 */
znetdkMobile.list.setCustomSortCriteria = function (sortCriteria, defaultSortCriterium) {
    if (this.element instanceof jQuery === false) {
        znetdkMobile.log.error('List is not instantiated!');
        return false;
    }
    if (sortCriteria === null || typeof sortCriteria !== 'object') {
        znetdkMobile.log.error('Not a JS object!');
        return false;
    }
    if (Object.keys(sortCriteria).length === 0) {
        znetdkMobile.log.error('The sort criteria value object is empty!');
        return false;
    }
    this.customSortCriteria = sortCriteria;
    if (typeof defaultSortCriterium === 'string') {
        if (sortCriteria.hasOwnProperty(defaultSortCriterium)) {
            this.defaultCustomSortCriterium = defaultSortCriterium;
        } else {
            znetdkMobile.log.error('The default sort criterium does not match any sort criteria!');
            return false;
        }
    }
    return true;
};

/**
 * Handle the click events on the row edit buttons by triggering a
 * 'afterclickeditbutton' event that is then catched for displaying the edit
 * modal dialog
 */
znetdkMobile.list.events.handleEdit = function () {
    var $this = this,
            eventName = 'click.znetdkmobile_list';
    znetdkMobile.content.getContainer().on(eventName, 'ul[data-zdk-load] a.edit', function (event) {
        var listElement = $(this).closest('ul'),
                rowElement = $(this).closest('li'),
                rowId = rowElement.data('id');
        if (typeof rowId === 'undefined') {
            znetdkMobile.log.warn("The 'data-id' attribute is missing!");
            return false;
        }
        listElement.triggerHandler($this.afterClickEditButtonName, [rowId]);
        event.preventDefault();
    });
};

/**
 * Handle the scroll events of the window for loading the list's rows.
 * This method is used to simulate the infinite scroll mechanism
 */
znetdkMobile.list.events.handleScroll = function () {
    var $this = this,
            eventName = 'scroll.znetdkmobile_list';
    $(window).on(eventName, function () {
        var displayedView = znetdkMobile.content.getDisplayedView();
        if (displayedView.length !== 1) {
            return false;
        }
        var listElement = displayedView.find('ul[data-zdk-load]');
        if (listElement.length !== 1) {
            return false;
        }
        listElement.triggerHandler($this.onContentScrollName);
    });
};

//*********************** AUTOCOMPLETE PUBLIC METHODS **************************

/**
 * Instantiate a new autocomplete object from the specified input field element
 * @param {String} inputSelector The selector of the input field as a jQuery
 * text selector
 * @param {Object} controllerAction The controller and the action to call
 * remotely to retrieve the autocompletion suggestions
 * @param {function} onSelect A function to call back when a suggestion has been
 * choosen by the user
 * @param {function} renderCallback Optional function called to customize the
 * display of each suggestion. This function is called with the suggestion's
 * data as parameter and must return the new label in text or HTML format.
 * @returns {Object|null} The instantiated autocomplete object or null if the
 * instantiation failed
 */
znetdkMobile.autocomplete.make = function (inputSelector, controllerAction,
        onSelect, renderCallback) {
    var inputElement = $(inputSelector);
    if (inputElement.length !== 1) {
        znetdkMobile.log.error('Autocomplete element is empty or multiple!');
        return null;
    }
    if (!inputElement.is('input')) {
        znetdkMobile.log.error('Must be an input element!', inputElement);
        return null;
    }
    if (inputElement.attr('type') !== 'search') {
        znetdkMobile.log.error("The input type must be 'search'!", inputElement);
        return null;
    }
    if (typeof controllerAction === 'undefined') {
        znetdkMobile.log.error("The controller action is missing!");
        return null;
    }
    if (controllerAction === null || typeof controllerAction !== 'object') {
        znetdkMobile.log.error("The controller action must be an object!", controllerAction);
        return null;
    }
    if (!controllerAction.hasOwnProperty('controller')) {
        znetdkMobile.log.error("The property 'controller' is missing!", controllerAction);
        return null;
    }
    if (!controllerAction.hasOwnProperty('action')) {
        znetdkMobile.log.error("The property 'action' is missing!", controllerAction);
        return null;
    }
    // HTML5 autocomplete attribute set to 'off'
    inputElement.attr('autocomplete', 'off');
    var autocompleteObject = Object.create(this);
    autocompleteObject.element = inputElement;
    autocompleteObject.remoteActions = {
        autocomplete: {
            controller: controllerAction.controller,
            action: controllerAction.action
        }
    };
    var list = autocompleteObject.element.next('ul.autocomplete'),
            timeout;
    if (list.length === 1) {
        list.remove();
    }
    list = $(autocompleteObject.listTemplate);
    autocompleteObject.element.after(list);

    _registerInputEvents();
    _registerSelectSuggestionEvents();
    return autocompleteObject;

    // PRIVATE METHODS
    function _registerInputEvents() {
        var eventName = 'input.znetdkmobile_autocomplete';
        autocompleteObject.element.off(eventName);
        autocompleteObject.element.on(eventName, function (event) {
            var queryString = $(this).val();
            _removeSuggestions();
            if (queryString.length < autocompleteObject.minStringLength) {
                return; // Number of characters insufficient
            }
            if (timeout) {
                window.clearTimeout(timeout);
            }
            timeout = window.setTimeout(function () {
                _showSuggestions(queryString);
            }, autocompleteObject.delay);
        });
    }
    function _showSuggestions(queryString) {
        znetdkMobile.ajax.request({
            control: controllerAction.controller,
            action: controllerAction.action,
            data: {query: queryString},
            callback: function (response) {
                _removeSuggestions();
                if (autocompleteObject.element.is(':focus') && response.length > 0) {
                    $.each(response, function () {
                        if (typeof this.label === 'undefined') {
                            znetdkMobile.log.warn("'label' property is missing in autocomplete suggestions!", this, 'error');
                            return false; // Break
                        }
                        var itemLabel = this.label,
                            item = $(autocompleteObject.itemTemplate),
                            htmlLabel = itemLabel.replace(new RegExp(queryString, 'gi'), '<strong>$&</strong>');
                        if (typeof renderCallback === 'function') {
                            itemLabel = renderCallback(this);
                            htmlLabel = itemLabel.replace(this.label, htmlLabel);
                        }
                        item.html(htmlLabel);
                        item.data('item', this);
                        list.append(item);
                    });
                    list.removeClass('w3-hide');
                    _registerClickOutOfTheAutocomplete();
                    _registerKeyboardAction();
                }
            }
        });
    }
    function _removeSuggestions() {
        list.empty();
        list.addClass('w3-hide');
        _registerClickOutOfTheAutocomplete(true);
        _registerKeyboardAction(true);
    }
    function _registerSelectSuggestionEvents() {
        var eventName = 'click.znetdkmobile_autocomplete';
        list.off(eventName);
        list.on(eventName, 'li', function () {
            var item = $(this).data('item'),
                    doSelection = true;
            if (typeof onSelect === 'function') {
                doSelection = onSelect.call(autocompleteObject.element, item);
            }
            if (doSelection !== false) {
                autocompleteObject.element.val(item.label);
            }
            _removeSuggestions();
        });
    }
    function _registerClickOutOfTheAutocomplete(unregisterOnly) {
        var eventName = 'click.znetdkmobile_autocomplete_close';
        $(document).off(eventName);
        if (unregisterOnly === true) {
            return false;
        }
        $(document).on(eventName, ':not(.autocomplete)', function () {
            _removeSuggestions();
        });
        return true;
    }
    function _registerKeyboardAction(unregisterOnly) {
        var eventName = 'keydown.znetdkmobile_autocomplete';
        autocompleteObject.element.off(eventName);
        if (unregisterOnly === true) {
            return false;
        }
        autocompleteObject.element.on(eventName, function (event) {
            if (event.keyCode === 40) { // DOWN key is pressed
                _setItemActive('down');
            } else if (event.keyCode === 38) { // UP key is pressed
                _setItemActive('up');
            } else if (event.keyCode === 13) { // ENTER key is pressed
                event.preventDefault();
                list.find('li.' + autocompleteObject.selectedItemCssClass).click();
            } else if (event.keyCode === 9) { // TAB key is pressed
                _removeSuggestions();
            }
        });
    }
    function _setItemActive(direction) {
        var selectedItem = list.find('li.' + autocompleteObject.selectedItemCssClass);
        if (selectedItem.length === 0) { // No item selected
            list.find('li').first().addClass(autocompleteObject.selectedItemCssClass);
        } else if (direction === 'up') {
            let prevItem = selectedItem.prev();
            selectedItem.removeClass(autocompleteObject.selectedItemCssClass);

            if (prevItem.length === 1) {
                prevItem.addClass(autocompleteObject.selectedItemCssClass);
            } else {
                list.find('li').last().addClass(autocompleteObject.selectedItemCssClass);
            }
        } else if (direction === 'down') {
            let nextItem = selectedItem.next();
            selectedItem.removeClass(autocompleteObject.selectedItemCssClass);
            if (nextItem.length === 1) {
                nextItem.addClass(autocompleteObject.selectedItemCssClass);
            } else {
                list.find('li').first().addClass(autocompleteObject.selectedItemCssClass);
            }
        }
    }
};

//*********************** SERVICE WORKER PUBLIC METHODS ************************

/**
 * Register the service worker of the application
 */
znetdkMobile.serviceWorker.register = function () {
    var scriptUrl = $('body').data('service-worker-url');
    if (scriptUrl.length === 0) {
        return false;
    }
    if ('serviceWorker' in navigator) {
        // Register a service worker hosted at the root of the
        // site using a more restrictive scope.
        navigator.serviceWorker.register(scriptUrl).then(undefined, function (error) {
            znetdkMobile.log.error("'" + scriptUrl + "': " + error.message);
            return false;
        });
    } else {
        znetdkMobile.log.warn('Service workers are not supported.');
        return false;
    }
    return true;
};

//**************************** FILE PUBLIC METHODS *****************************

/**
 * Download from the specified URL the specified resource (picture or document)
 * and display it.
 * @param {String} url The url of the resource to download
 */
znetdkMobile.file.display = function (url) {
    // First check if the user session has not expired
    znetdkMobile.ajax.request({controller: 'security', action: 'isconnected'});
    // Download and display the picture or document
    var link = document.createElement("a");
    link.href = url;
    link.rel = 'noopener';
    link.target = '_blank';
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
};
